// Datei echu.c
/* ------------------------------------------------------------------------
"echu" like "echo for Umgebungsvariablen" ("echo for environment variables")
Displays the content of environment variables in a legible form.

Should normally be called with one parameter:
The name of an evironment variable (e.g. PATH, CLASSPATH, HOME etc.)
Outputs the content of that variable "in a legible form", i.e.
after each semicolon ';' in the content a new line is started.
This is useful for variables like PATH, CLASSPATH and PATHEXT.
For other environment variables, e.g. HOME, the output of
$ echu home
is similar to the output of
$ echo %HOME%

Params for echu may be in any case (e.g. home or HOME or HoMe etc.

If echu is called without parameters, the content of PATH is output.
If echu is called with p (or c) as parameter, the content of PATH
(resp. CLASSPATH) is output.
------------------------------------------------------------------------ */
#include <stdio.h>   // I/O with bytestreams, printf, scanf, sprintf, ...
                     // fopen, fclose, fflush, fgets, fputs, fread, fwrite, ...
#include <stdlib.h>  // atof, atoi, atol, rand, malloc, calloc, free, exit,
                     // abs, div, getenv, system, bsearch, qsort, ...
#include <string.h>  // memcopy, memmove, strlen, strcpy, strcat, ...
#include <ctype.h>   // isalpha, isdigit, islower, isupper, tolower, ...

int main (int argc, char * argv[]) {

   // All variables are defined here in order to give older
   // C-compilers (like the Borland bcc32) a chance to compile this:
	const char * const MINUS_LINE =
	   "------------------------------------------------\n";
   char * varName;      // Name of environment variable to be displayed
	char * p;            // A pointer
   char * varContent;   // Content of the variable to be displayed
   char newlineOrBlank;

   printf(MINUS_LINE);

   if (argc-1 > 1) {
      printf("Program echu has been called with %d parameters.\n", argc-1);
      printf("It should be called with 0 or 1 param(s), e.g.: \n");
      printf("$ echu        // 0 params, defaults to PATH     \n");
      printf("$ echu PATH   // 1 param,  name of a variable   \n");
      printf("$ echu P      // 1 param,  abbrev. for PATH     \n");
      printf("$ echu C      // 1 param,  abbrev. for CLASSPATH\n");
      printf(MINUS_LINE);
      return 1;
   }

   if (argc == 1) {
      varName = "PATH";
   } else {
		varName = argv[1];
	}

	p = varName;
	while (*p != 0) {
		*p = toupper(*p);
		p++;
	}

	if (strcmp(varName, "P")==0) varName="PATH";
	if (strcmp(varName, "C")==0) varName="CLASSPATH";

	varContent = (char *) getenv(varName);

	if (varContent == NULL) {
      printf("Environment variable %s is empty!\n", varName);
      printf(MINUS_LINE);
      return 0;
	}

   printf("Content of environment variable %s\n", varName);
   printf(MINUS_LINE);
   p = varContent;
   newlineOrBlank = ' ';
   while (*p!=0) {
      if (*p==';') {
         printf(";\n");
         newlineOrBlank = ' ';
      } else {
         printf("%c", *p);
         newlineOrBlank = '\n';
      }
      p++;
   }
   printf("%c", newlineOrBlank);

   return 0;
}  // main
