// Datei IntTypes03.c
/* ------------------------------------------------------------------------
Was Typnamen wie short, int oder long bedeuten (vor allem :  wieviele Bits
Werte dieser Typen belegen) wird durch keinen C-Standard festgelegt.

Der C-Standard C99 regt aber an, dass in der Kopfdatei <stdint.h>
einige der folgenden Typnamen

       int8_t,        int12_t,        int16_t, ...
      uint8_t,       uint12_t,       uint16_t, ...
 int_least8_t,  int_least12_t,  int_least16_t, ...
uint_least8_t, uint_least12_t, uint_least16_t, ...
  int_fast8_t,   int_fast12_t,   int_fast16_t, ...
 uint_fast8_t,  uint_fast12_t,  uint_fast16_t, ...

mit standardisierten Bedeutungen vereinbart werden. Jeder solche Name
soll einen Ganzzahltyp bezeichnen, dessen negative Werte im Zweier-
komplement dargestellt werden. Darueber hinaus sollen diese Typen
folgende Eigenschaften haben:

       int8_t : Breite exakt 8 Bits, mit  Vorzeichen
      uint8_t : Breite exakt 8 Bits, ohne Vorzeichen
 int_least8_t : Schmalster Typ mit 8 Bits, mit Vorzeichen
uint_least8_t : Schmalster Typ mit 8 Bits, ohne Vorzeichen
  int_fast8_t : Typ mit mindestens 8 Bits, dessen Werte der Ausfuehrer
                besonders schnell bearbeiten kann, mit Vorzeichen.
 uint_fast8_t : Typ mit mindestens 8 Bits, dessen Werte der Ausfuehrer
                besonders schnell bearbeiten kann, ohne Vorzeichen.

Damit man Werte dieser Typen mit printf formatieren und ausgeben,
und mit scanf formatiert einlesen kann, sind (in der Kompfdatei
<inttypes.h>) entsprechende string-Makros namens PRId8, PRIdLEAST8,
PRIdFAST8 etc. definiert.

Fuer die anderen Breiten 12, 16, ... etc. entsprechend.
------------------------------------------------------------------------ */
#include <stdint.h>  // int8_t, uint8_t, int_least8_t, uint_least8_t,
                     // int_fast8_t, intptr_t, uintptr_t, intmax_t, ...
#include <inttypes.h>// PRId8, PRIdLEAST8, PRIdFAST8, ...
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...

int main() {
   printf("IntTypes03 :  Jetzt geht es los!\n");
   printf("=============================================\n");
   #ifndef PRId8
   printf("Den Typ int8_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int8_t etc.: \n");

         int8_t       int8_vn =       INT8_MIN;
         int8_t       int8_vp =       INT8_MAX;
        uint8_t      uint8_vp =      UINT8_MAX;
   int_least8_t int_least8_vn = INT_LEAST8_MIN;
   int_least8_t int_least8_vp = INT_LEAST8_MAX;
    int_fast8_t  int_fast8_vn =  INT_FAST8_MIN;
    int_fast8_t  int_fast8_vp =  INT_FAST8_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int8_t) : %21d \n", sizeof(      int8_t));
   printf("sizeof(     uint8_t) : %21d \n", sizeof(     uint8_t));
   printf("sizeof(int_least8_t) : %21d \n", sizeof(int_least8_t));
   printf("sizeof( int_fast8_t) : %21d \n", sizeof( int_fast8_t));
   printf("---------------------+-----------------------\n");
   printf("             int8_vn : %+21"PRId8          " \n",       int8_vn);
   printf("             int8_vp : %+21"PRId8          " \n",       int8_vp);
   printf("            uint8_vp : %+21"PRIu8          " \n",      uint8_vp);
   printf("       int_least8_vn : %+21"PRIdLEAST8     " \n", int_least8_vn);
   printf("       int_least8_vp : %+21"PRIdLEAST8     " \n", int_least8_vp);
   printf("        int_fast8_vn : %+21"PRIdFAST8      " \n",  int_fast8_vn);
   printf("        int_fast8_vp : %+21"PRIdFAST8      " \n",  int_fast8_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId8: %s, PRIdLEAST8: %s, PRIdFAST8: %s\n",
           PRId8,     PRIdLEAST8,     PRIdFAST8);
   printf("PRIo8: %s, PRIoLEAST8: %s, PRIoFAST8: %s\n",
           PRIo8,     PRIoLEAST8,     PRIdFAST8);
   printf("PRIu8: %s, PRIuLEAST8: %s, PRIuFAST8: %s\n",
           PRIu8,     PRIuLEAST8,     PRIuFAST8);
   printf("PRIx8: %s, PRIxLEAST8: %s, PRIxFAST8: %s\n",
           PRIx8,     PRIxLEAST8,     PRIxFAST8);
   printf("PRIX8: %s, PRIXLEAST8: %s, PRIXFAST8: %s\n",
           PRIX8,     PRIXLEAST8,     PRIXFAST8);
   #endif // PRId8
   printf("=============================================\n");
   #ifndef PRId12
   printf("Den Typ int12_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int12_t etc.: \n");

         int12_t       int12_vn =       INT12_MIN;
         int12_t       int12_vp =       INT12_MAX;
        uint12_t      uint12_vp =      UINT12_MAX;
   int_least12_t int_least12_vn = INT_LEAST12_MIN;
   int_least12_t int_least12_vp = INT_LEAST12_MAX;
    int_fast12_t  int_fast12_vn =  INT_FAST12_MIN;
    int_fast12_t  int_fast12_vp =  INT_FAST12_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int12_t) : %21d \n", sizeof(      int12_t));
   printf("sizeof(     uint12_t) : %21d \n", sizeof(     uint12_t));
   printf("sizeof(int_least12_t) : %21d \n", sizeof(int_least12_t));
   printf("sizeof( int_fast12_t) : %21d \n", sizeof( int_fast12_t));
   printf("---------------------+-----------------------\n");
   printf("             int12_vn : %+21"PRId12        " \n",       int12_vn);
   printf("             int12_vp : %+21"PRId12        " \n",       int12_vp);
   printf("            uint12_vp : %+21"PRIu12        " \n",      uint12_vp);
   printf("       int_least12_vn : %+21"PRIdLEAST12   " \n", int_least12_vn);
   printf("       int_least12_vp : %+21"PRIdLEAST12   " \n", int_least12_vp);
   printf("        int_fast12_vn : %+21"PRIdFAST12    " \n",  int_fast12_vn);
   printf("        int_fast12_vp : %+21"PRIdFAST12    " \n",  int_fast12_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId12: %s, PRIdLEAST12: %s, PRIdFAST12: %s\n",
           PRId12,     PRIdLEAST12,     PRIdFAST12);
   printf("PRIo12: %s, PRIoLEAST12: %s, PRIoFAST12: %s\n",
           PRIo12,     PRIoLEAST12,     PRIdFAST12);
   printf("PRIu12: %s, PRIuLEAST12: %s, PRIuFAST12: %s\n",
           PRIu12,     PRIuLEAST12,     PRIuFAST12);
   printf("PRIx12: %s, PRIxLEAST12: %s, PRIxFAST12: %s\n",
           PRIx12,     PRIxLEAST12,     PRIxFAST12);
   printf("PRIX12: %s, PRIXLEAST12: %s, PRIXFAST12: %s\n",
           PRIX12,     PRIXLEAST12,     PRIXFAST12);

   #endif // PRId12
   printf("=============================================\n");
   #ifndef PRId16
   printf("Den Typ int16_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int16_t etc.: \n");

         int16_t       int16_vn =       INT16_MIN;
         int16_t       int16_vp =       INT16_MAX;
        uint16_t      uint16_vp =      UINT16_MAX;
   int_least16_t int_least16_vn = INT_LEAST16_MIN;
   int_least16_t int_least16_vp = INT_LEAST16_MAX;
    int_fast16_t  int_fast16_vn =  INT_FAST16_MIN;
    int_fast16_t  int_fast16_vp =  INT_FAST16_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int16_t) : %21d \n", sizeof(      int16_t));
   printf("sizeof(     uint16_t) : %21d \n", sizeof(     uint16_t));
   printf("sizeof(int_least16_t) : %21d \n", sizeof(int_least16_t));
   printf("sizeof( int_fast16_t) : %21d \n", sizeof( int_fast16_t));
   printf("---------------------+-----------------------\n");
   printf("             int16_vn : %+21"PRId16        " \n",       int16_vn);
   printf("             int16_vp : %+21"PRId16        " \n",       int16_vp);
   printf("            uint16_vp : %+21"PRIu16        " \n",      uint16_vp);
   printf("       int_least16_vn : %+21"PRIdLEAST16   " \n", int_least16_vn);
   printf("       int_least16_vp : %+21"PRIdLEAST16   " \n", int_least16_vp);
   printf("        int_fast16_vn : %+21"PRIdFAST16    " \n",  int_fast16_vn);
   printf("        int_fast16_vp : %+21"PRIdFAST16    " \n",  int_fast16_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId16: %s, PRIdLEAST16: %s, PRIdFAST16: %s\n",
           PRId16,     PRIdLEAST16,     PRIdFAST16);
   printf("PRIo16: %s, PRIoLEAST16: %s, PRIoFAST16: %s\n",
           PRIo16,     PRIoLEAST16,     PRIdFAST16);
   printf("PRIu16: %s, PRIuLEAST16: %s, PRIuFAST16: %s\n",
           PRIu16,     PRIuLEAST16,     PRIuFAST16);
   printf("PRIx16: %s, PRIxLEAST16: %s, PRIxFAST16: %s\n",
           PRIx16,     PRIxLEAST16,     PRIxFAST16);
   printf("PRIX16: %s, PRIXLEAST16: %s, PRIXFAST16: %s\n",
           PRIX16,     PRIXLEAST16,     PRIXFAST16);

   #endif // PRId16
   printf("=============================================\n");
   #ifndef PRId24
   printf("Den Typ int24_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int24_t etc.: \n");

         int24_t       int24_vn =       INT24_MIN;
         int24_t       int24_vp =       INT24_MAX;
        uint24_t      uint24_vp =      UINT24_MAX;
   int_least24_t int_least24_vn = INT_LEAST24_MIN;
   int_least24_t int_least24_vp = INT_LEAST24_MAX;
    int_fast24_t  int_fast24_vn =  INT_FAST24_MIN;
    int_fast24_t  int_fast24_vp =  INT_FAST24_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int24_t) : %21d \n", sizeof(      int24_t));
   printf("sizeof(     uint24_t) : %21d \n", sizeof(     uint24_t));
   printf("sizeof(int_least24_t) : %21d \n", sizeof(int_least24_t));
   printf("sizeof( int_fast24_t) : %21d \n", sizeof( int_fast24_t));
   printf("---------------------+-----------------------\n");
   printf("             int24_vn : %+21"PRId24        " \n",       int24_vn);
   printf("             int24_vp : %+21"PRId24        " \n",       int24_vp);
   printf("            uint24_vp : %+21"PRIu24        " \n",      uint24_vp);
   printf("       int_least24_vn : %+21"PRIdLEAST24   " \n", int_least24_vn);
   printf("       int_least24_vp : %+21"PRIdLEAST24   " \n", int_least24_vp);
   printf("        int_fast24_vn : %+21"PRIdFAST24    " \n",  int_fast24_vn);
   printf("        int_fast24_vp : %+21"PRIdFAST24    " \n",  int_fast24_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId24: %s, PRIdLEAST24: %s, PRIdFAST24: %s\n",
           PRId24,     PRIdLEAST24,     PRIdFAST24);
   printf("PRIo24: %s, PRIoLEAST24: %s, PRIoFAST24: %s\n",
           PRIo24,     PRIoLEAST24,     PRIdFAST24);
   printf("PRIu24: %s, PRIuLEAST24: %s, PRIuFAST24: %s\n",
           PRIu24,     PRIuLEAST24,     PRIuFAST24);
   printf("PRIx24: %s, PRIxLEAST24: %s, PRIxFAST24: %s\n",
           PRIx24,     PRIxLEAST24,     PRIxFAST24);
   printf("PRIX24: %s, PRIXLEAST24: %s, PRIXFAST24: %s\n",
           PRIX24,     PRIXLEAST24,     PRIXFAST24);

   #endif // PRId24
   printf("=============================================\n");
   #ifndef PRId32
   printf("Den Typ int32_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int32_t etc.: \n");

         int32_t       int32_vn =       INT32_MIN;
         int32_t       int32_vp =       INT32_MAX;
        uint32_t      uint32_vp =      UINT32_MAX;
   int_least32_t int_least32_vn = INT_LEAST32_MIN;
   int_least32_t int_least32_vp = INT_LEAST32_MAX;
    int_fast32_t  int_fast32_vn =  INT_FAST32_MIN;
    int_fast32_t  int_fast32_vp =  INT_FAST32_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int32_t) : %21d \n", sizeof(      int32_t));
   printf("sizeof(     uint32_t) : %21d \n", sizeof(     uint32_t));
   printf("sizeof(int_least32_t) : %21d \n", sizeof(int_least32_t));
   printf("sizeof( int_fast32_t) : %21d \n", sizeof( int_fast32_t));
   printf("---------------------+-----------------------\n");
   printf("             int32_vn : %+21"PRId32        " \n",       int32_vn);
   printf("             int32_vp : %+21"PRId32        " \n",       int32_vp);
   printf("            uint32_vp : %+21"PRIu32        " \n",      uint32_vp);
   printf("       int_least32_vn : %+21"PRIdLEAST32   " \n", int_least32_vn);
   printf("       int_least32_vp : %+21"PRIdLEAST32   " \n", int_least32_vp);
   printf("        int_fast32_vn : %+21"PRIdFAST32    " \n",  int_fast32_vn);
   printf("        int_fast32_vp : %+21"PRIdFAST32    " \n",  int_fast32_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId32: %s, PRIdLEAST32: %s, PRIdFAST32: %s\n",
           PRId32,     PRIdLEAST32,     PRIdFAST32);
   printf("PRIo32: %s, PRIoLEAST32: %s, PRIoFAST32: %s\n",
           PRIo32,     PRIoLEAST32,     PRIdFAST32);
   printf("PRIu32: %s, PRIuLEAST32: %s, PRIuFAST32: %s\n",
           PRIu32,     PRIuLEAST32,     PRIuFAST32);
   printf("PRIx32: %s, PRIxLEAST32: %s, PRIxFAST32: %s\n",
           PRIx32,     PRIxLEAST32,     PRIxFAST32);
   printf("PRIX32: %s, PRIXLEAST32: %s, PRIXFAST32: %s\n",
           PRIX32,     PRIXLEAST32,     PRIXFAST32);

   #endif // PRId32
   printf("=============================================\n");
   #ifndef PRId36
   printf("Den Typ int36_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int36_t etc.: \n");

         int36_t       int36_vn =       INT36_MIN;
         int36_t       int36_vp =       INT36_MAX;
        uint36_t      uint36_vp =      UINT36_MAX;
   int_least36_t int_least36_vn = INT_LEAST36_MIN;
   int_least36_t int_least36_vp = INT_LEAST36_MAX;
    int_fast36_t  int_fast36_vn =  INT_FAST36_MIN;
    int_fast36_t  int_fast36_vp =  INT_FAST36_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int36_t) : %21d \n", sizeof(      int36_t));
   printf("sizeof(     uint36_t) : %21d \n", sizeof(     uint36_t));
   printf("sizeof(int_least36_t) : %21d \n", sizeof(int_least36_t));
   printf("sizeof( int_fast36_t) : %21d \n", sizeof( int_fast36_t));
   printf("---------------------+-----------------------\n");
   printf("             int36_vn : %+21"PRId36        " \n",       int36_vn);
   printf("             int36_vp : %+21"PRId36        " \n",       int36_vp);
   printf("            uint36_vp : %+21"PRIu36        " \n",      uint36_vp);
   printf("       int_least36_vn : %+21"PRIdLEAST36   " \n", int_least36_vn);
   printf("       int_least36_vp : %+21"PRIdLEAST36   " \n", int_least36_vp);
   printf("        int_fast36_vn : %+21"PRIdFAST36    " \n",  int_fast36_vn);
   printf("        int_fast36_vp : %+21"PRIdFAST36    " \n",  int_fast36_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId36: %s, PRIdLEAST36: %s, PRIdFAST36: %s\n",
           PRId36,     PRIdLEAST36,     PRIdFAST36);
   printf("PRIo36: %s, PRIoLEAST36: %s, PRIoFAST36: %s\n",
           PRIo36,     PRIoLEAST36,     PRIdFAST36);
   printf("PRIu36: %s, PRIuLEAST36: %s, PRIuFAST36: %s\n",
           PRIu36,     PRIuLEAST36,     PRIuFAST36);
   printf("PRIx36: %s, PRIxLEAST36: %s, PRIxFAST36: %s\n",
           PRIx36,     PRIxLEAST36,     PRIxFAST36);
   printf("PRIX36: %s, PRIXLEAST36: %s, PRIXFAST36: %s\n",
           PRIX36,     PRIXLEAST36,     PRIXFAST36);

   #endif // PRId36
   printf("=============================================\n");
   #ifndef PRId48
   printf("Den Typ int48_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int48_t etc.: \n");

         int48_t       int48_vn =       INT48_MIN;
         int48_t       int48_vp =       INT48_MAX;
        uint48_t      uint48_vp =      UINT48_MAX;
   int_least48_t int_least48_vn = INT_LEAST48_MIN;
   int_least48_t int_least48_vp = INT_LEAST48_MAX;
    int_fast48_t  int_fast48_vn =  INT_FAST48_MIN;
    int_fast48_t  int_fast48_vp =  INT_FAST48_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int48_t) : %21d \n", sizeof(      int48_t));
   printf("sizeof(     uint48_t) : %21d \n", sizeof(     uint48_t));
   printf("sizeof(int_least48_t) : %21d \n", sizeof(int_least48_t));
   printf("sizeof( int_fast48_t) : %21d \n", sizeof( int_fast48_t));
   printf("---------------------+-----------------------\n");
   printf("             int48_vn : %+21"PRId48        " \n",       int48_vn);
   printf("             int48_vp : %+21"PRId48        " \n",       int48_vp);
   printf("            uint48_vp : %+21"PRIu48        " \n",      uint48_vp);
   printf("       int_least48_vn : %+21"PRIdLEAST48   " \n", int_least48_vn);
   printf("       int_least48_vp : %+21"PRIdLEAST48   " \n", int_least48_vp);
   printf("        int_fast48_vn : %+21"PRIdFAST48    " \n",  int_fast48_vn);
   printf("        int_fast48_vp : %+21"PRIdFAST48    " \n",  int_fast48_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId48: %s, PRIdLEAST48: %s, PRIdFAST48: %s\n",
           PRId48,     PRIdLEAST48,     PRIdFAST48);
   printf("PRIo48: %s, PRIoLEAST48: %s, PRIoFAST48: %s\n",
           PRIo48,     PRIoLEAST48,     PRIdFAST48);
   printf("PRIu48: %s, PRIuLEAST48: %s, PRIuFAST48: %s\n",
           PRIu48,     PRIuLEAST48,     PRIuFAST48);
   printf("PRIx48: %s, PRIxLEAST48: %s, PRIxFAST48: %s\n",
           PRIx48,     PRIxLEAST48,     PRIxFAST48);
   printf("PRIX48: %s, PRIXLEAST48: %s, PRIXFAST48: %s\n",
           PRIX48,     PRIXLEAST48,     PRIXFAST48);

   #endif // PRId48
   printf("=============================================\n");
   #ifndef PRId64
   printf("Den Typ int64_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int64_t etc.: \n");

         int64_t       int64_vn =       INT64_MIN;
         int64_t       int64_vp =       INT64_MAX;
        uint64_t      uint64_vp =      UINT64_MAX;
   int_least64_t int_least64_vn = INT_LEAST64_MIN;
   int_least64_t int_least64_vp = INT_LEAST64_MAX;
    int_fast64_t  int_fast64_vn =  INT_FAST64_MIN;
    int_fast64_t  int_fast64_vp =  INT_FAST64_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int64_t) : %21d \n", sizeof(      int64_t));
   printf("sizeof(     uint64_t) : %21d \n", sizeof(     uint64_t));
   printf("sizeof(int_least64_t) : %21d \n", sizeof(int_least64_t));
   printf("sizeof( int_fast64_t) : %21d \n", sizeof( int_fast64_t));
   printf("---------------------+-----------------------\n");
   printf("             int64_vn : %+21"PRId64        " \n",       int64_vn);
   printf("             int64_vn : %+21"PRId64        " \n",       int64_vn);

   printf("             int64_vp : %+21"PRId64        " \n",       int64_vp);
   printf("             int64_vp : %+21"PRId64        " \n",       int64_vp);

   printf("            uint64_vp : %+21"PRIu64        " \n",      uint64_vp);
   printf("       int_least64_vn : %+21"PRIdLEAST64   " \n", int_least64_vn);
   printf("       int_least64_vp : %+21"PRIdLEAST64   " \n", int_least64_vp);
   printf("        int_fast64_vn : %+21"PRIdFAST64    " \n",  int_fast64_vn);
   printf("        int_fast64_vp : %+21"PRIdFAST64    " \n",  int_fast64_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId64: %s, PRIdLEAST64: %s, PRIdFAST64: %s\n",
           PRId64,     PRIdLEAST64,     PRIdFAST64);
   printf("PRIo64: %s, PRIoLEAST64: %s, PRIoFAST64: %s\n",
           PRIo64,     PRIoLEAST64,     PRIdFAST64);
   printf("PRIu64: %s, PRIuLEAST64: %s, PRIuFAST64: %s\n",
           PRIu64,     PRIuLEAST64,     PRIuFAST64);
   printf("PRIx64: %s, PRIxLEAST64: %s, PRIxFAST64: %s\n",
           PRIx64,     PRIxLEAST64,     PRIxFAST64);
   printf("PRIX64: %s, PRIXLEAST64: %s, PRIXFAST64: %s\n",
           PRIX64,     PRIXLEAST64,     PRIXFAST64);

   #endif // PRId64
   printf("=============================================\n");
   #ifndef PRId96
   printf("Den Typ int96_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int96_t etc.: \n");

         int96_t       int96_vn =       INT96_MIN;
         int96_t       int96_vp =       INT96_MAX;
        uint96_t      uint96_vp =      UINT96_MAX;
   int_least96_t int_least96_vn = INT_LEAST96_MIN;
   int_least96_t int_least96_vp = INT_LEAST96_MAX;
    int_fast96_t  int_fast96_vn =  INT_FAST96_MIN;
    int_fast96_t  int_fast96_vp =  INT_FAST96_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int96_t) : %21d \n", sizeof(      int96_t));
   printf("sizeof(     uint96_t) : %21d \n", sizeof(     uint96_t));
   printf("sizeof(int_least96_t) : %21d \n", sizeof(int_least96_t));
   printf("sizeof( int_fast96_t) : %21d \n", sizeof( int_fast96_t));
   printf("---------------------+-----------------------\n");
   printf("             int96_vn : %+21"PRId96        " \n",       int96_vn);
   printf("             int96_vp : %+21"PRId96        " \n",       int96_vp);
   printf("            uint96_vp : %+21"PRIu96        " \n",      uint96_vp);
   printf("       int_least96_vn : %+21"PRIdLEAST96   " \n", int_least96_vn);
   printf("       int_least96_vp : %+21"PRIdLEAST96   " \n", int_least96_vp);
   printf("        int_fast96_vn : %+21"PRIdFAST96    " \n",  int_fast96_vn);
   printf("        int_fast96_vp : %+21"PRIdFAST96    " \n",  int_fast96_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId96: %s, PRIdLEAST96: %s, PRIdFAST96: %s\n",
           PRId96,     PRIdLEAST96,     PRIdFAST96);
   printf("PRIo96: %s, PRIoLEAST96: %s, PRIoFAST96: %s\n",
           PRIo96,     PRIoLEAST96,     PRIdFAST96);
   printf("PRIu96: %s, PRIuLEAST96: %s, PRIuFAST96: %s\n",
           PRIu96,     PRIuLEAST96,     PRIuFAST96);
   printf("PRIx96: %s, PRIxLEAST96: %s, PRIxFAST96: %s\n",
           PRIx96,     PRIxLEAST96,     PRIxFAST96);
   printf("PRIX96: %s, PRIXLEAST96: %s, PRIXFAST96: %s\n",
           PRIX96,     PRIXLEAST96,     PRIXFAST96);

   #endif // PRId96
   printf("=============================================\n");
   #ifndef PRId128
   printf("Den Typ int128_t kennt dieser Compiler nicht!\n");
   #else
   printf("Der Typ int128_t etc.: \n");

         int128_t       int128_vn =       INT128_MIN;
         int128_t       int128_vp =       INT128_MAX;
        uint128_t      uint128_vp =      UINT128_MAX;
   int_least128_t int_least128_vn = INT_LEAST128_MIN;
   int_least128_t int_least128_vp = INT_LEAST128_MAX;
    int_fast128_t  int_fast128_vn =  INT_FAST128_MIN;
    int_fast128_t  int_fast128_vp =  INT_FAST128_MAX;

   printf("---------------------+-----------------------\n");
   printf("sizeof(      int128_t) : %21d \n", sizeof(      int128_t));
   printf("sizeof(     uint128_t) : %21d \n", sizeof(     uint128_t));
   printf("sizeof(int_least128_t) : %21d \n", sizeof(int_least128_t));
   printf("sizeof( int_fast128_t) : %21d \n", sizeof( int_fast128_t));
   printf("---------------------+-----------------------\n");
   printf("             int128_vn : %+21"PRId128      " \n",       int128_vn);
   printf("             int128_vp : %+21"PRId128      " \n",       int128_vp);
   printf("            uint128_vp : %+21"PRIu128      " \n",      uint128_vp);
   printf("       int_least128_vn : %+21"PRIdLEAST128 " \n", int_least128_vn);
   printf("       int_least128_vp : %+21"PRIdLEAST128 " \n", int_least128_vp);
   printf("        int_fast128_vn : %+21"PRIdFAST128  " \n",  int_fast128_vn);
   printf("        int_fast128_vp : %+21"PRIdFAST128  " \n",  int_fast128_vp);
   printf("---------------------+-----------------------\n");
   printf("PRId128: %s, PRIdLEAST128: %s, PRIdFAST128: %s\n",
           PRId128,     PRIdLEAST128,     PRIdFAST128);
   printf("PRIo128: %s, PRIoLEAST128: %s, PRIoFAST128: %s\n",
           PRIo128,     PRIoLEAST128,     PRIdFAST128);
   printf("PRIu128: %s, PRIuLEAST128: %s, PRIuFAST128: %s\n",
           PRIu128,     PRIuLEAST128,     PRIuFAST128);
   printf("PRIx128: %s, PRIxLEAST128: %s, PRIxFAST128: %s\n",
           PRIx128,     PRIxLEAST128,     PRIxFAST128);
   printf("PRIX128: %s, PRIXLEAST128: %s, PRIXFAST128: %s\n",
           PRIX128,     PRIXLEAST128,     PRIXFAST128);

   #endif // PRId128
   printf("=============================================\n");
   printf("IntTypes03 :  Das war's erstmal!\n");
   return 0;
} // main
/* ------------------------------------------------------------------------
Zweimal die Ausgabe des Programms IntTypes03

(compiliert mit dem gcc, v3.4.4):                (compiliert mit dem lcc-win32, v3.8):

IntTypes03 :  Jetzt geht es los!                 IntTypes03 :  Jetzt geht es los!
=============================================    =============================================
Der Typ int8_t etc.:                             Der Typ int8_t etc.:
---------------------+-----------------------    ---------------------+-----------------------
sizeof(      int8_t) :                     1     sizeof(      int8_t) :                     1
sizeof(     uint8_t) :                     1     sizeof(     uint8_t) :                     1
sizeof(int_least8_t) :                     1     sizeof(int_least8_t) :                     1
sizeof( int_fast8_t) :                     1     sizeof( int_fast8_t) :                     1
---------------------+-----------------------    ---------------------+-----------------------
             int8_vn :                  -128                  int8_vn :                  -128
             int8_vp :                  +127                  int8_vp :                  +127
            uint8_vp :                   255                 uint8_vp :                   255
       int_least8_vn :                  -128            int_least8_vn :                  -128
       int_least8_vp :                  +127            int_least8_vp :                  +127
        int_fast8_vn :                  -128             int_fast8_vn :                  -128
        int_fast8_vp :                  +127             int_fast8_vp :                  +127
---------------------+-----------------------    ---------------------+-----------------------
PRId8: d, PRIdLEAST8: d, PRIdFAST8: d            PRId8: d, PRIdLEAST8: d, PRIdFAST8: d
PRIo8: o, PRIoLEAST8: o, PRIoFAST8: d            PRIo8: o, PRIoLEAST8: o, PRIoFAST8: d
PRIu8: u, PRIuLEAST8: u, PRIuFAST8: u            PRIu8: u, PRIuLEAST8: u, PRIuFAST8: u
PRIx8: x, PRIxLEAST8: x, PRIxFAST8: x            PRIx8: x, PRIxLEAST8: x, PRIxFAST8: x
PRIX8: X, PRIXLEAST8: X, PRIXFAST8: X            PRIX8: X, PRIXLEAST8: X, PRIXFAST8: X
=============================================    =============================================
Den Typ int12_t kennt dieser Compiler nicht!     Den Typ int12_t kennt dieser Compiler nicht!
=============================================    =============================================
Der Typ int16_t etc.:                            Der Typ int16_t etc.:
---------------------+-----------------------    ---------------------+-----------------------
sizeof(      int16_t) :                     2    sizeof(      int16_t) :                     2
sizeof(     uint16_t) :                     2    sizeof(     uint16_t) :                     2
sizeof(int_least16_t) :                     2    sizeof(int_least16_t) :                     2
sizeof( int_fast16_t) :                     4    sizeof( int_fast16_t) :                     2
---------------------+-----------------------    ---------------------+-----------------------
             int16_vn :                -32768                 int16_vn :                -32768
             int16_vp :                +32767                 int16_vp :                +32767
            uint16_vp :                 65535                uint16_vp :                 65535
       int_least16_vn :                -32768           int_least16_vn :                -32768
       int_least16_vp :                +32767           int_least16_vp :                +32767
        int_fast16_vn :           -2147483648            int_fast16_vn :                -32768
        int_fast16_vp :           +2147483647            int_fast16_vp :                +32767
---------------------+-----------------------    ---------------------+-----------------------
PRId16: d, PRIdLEAST16: d, PRIdFAST16: ld        PRId16: d, PRIdLEAST16: d, PRIdFAST16: d
PRIo16: o, PRIoLEAST16: o, PRIoFAST16: ld        PRIo16: o, PRIoLEAST16: o, PRIoFAST16: d
PRIu16: u, PRIuLEAST16: u, PRIuFAST16: lu        PRIu16: u, PRIuLEAST16: u, PRIuFAST16: u
PRIx16: x, PRIxLEAST16: x, PRIxFAST16: lx        PRIx16: x, PRIxLEAST16: x, PRIxFAST16: x
PRIX16: X, PRIXLEAST16: X, PRIXFAST16: lX        PRIX16: X, PRIXLEAST16: X, PRIXFAST16: X
=============================================    =============================================
Den Typ int24_t kennt dieser Compiler nicht!     Den Typ int24_t kennt dieser Compiler nicht!
=============================================    =============================================
Der Typ int32_t etc.:                            Der Typ int32_t etc.:
---------------------+-----------------------    ---------------------+-----------------------
sizeof(      int32_t) :                     4    sizeof(      int32_t) :                     4
sizeof(     uint32_t) :                     4    sizeof(     uint32_t) :                     4
sizeof(int_least32_t) :                     4    sizeof(int_least32_t) :                     4
sizeof( int_fast32_t) :                     4    sizeof( int_fast32_t) :                     4
---------------------+-----------------------    ---------------------+-----------------------
             int32_vn :           -2147483648                 int32_vn :           -2147483648
             int32_vp :           +2147483647                 int32_vp :           +2147483647
            uint32_vp :            4294967295                uint32_vp :            4294967295
       int_least32_vn :           -2147483648           int_least32_vn :           -2147483648
       int_least32_vp :           +2147483647           int_least32_vp :           +2147483647
        int_fast32_vn :           -2147483648            int_fast32_vn :           -2147483648
        int_fast32_vp :           +2147483647            int_fast32_vp :           +2147483647
---------------------+-----------------------    ---------------------+-----------------------
PRId32: ld, PRIdLEAST32: ld, PRIdFAST32: ld      PRId32: d, PRIdLEAST32: d, PRIdFAST32: d
PRIo32: lo, PRIoLEAST32: lo, PRIoFAST32: ld      PRIo32: o, PRIoLEAST32: o, PRIoFAST32: d
PRIu32: lu, PRIuLEAST32: lu, PRIuFAST32: lu      PRIu32: u, PRIuLEAST32: u, PRIuFAST32: u
PRIx32: lx, PRIxLEAST32: lx, PRIxFAST32: lx      PRIx32: x, PRIxLEAST32: x, PRIxFAST32: x
PRIX32: lX, PRIXLEAST32: lX, PRIXFAST32: lX      PRIX32: X, PRIXLEAST32: X, PRIXFAST32: X
=============================================    =============================================
Den Typ int36_t kennt dieser Compiler nicht!     Den Typ int36_t kennt dieser Compiler nicht!
=============================================    =============================================
Den Typ int48_t kennt dieser Compiler nicht!     Den Typ int48_t kennt dieser Compiler nicht!
=============================================    =============================================
Der Typ int64_t etc.:                            Der Typ int64_t etc.:
---------------------+-----------------------    ---------------------+-----------------------
sizeof(      int64_t) :                     8    sizeof(      int64_t) :                     8
sizeof(     uint64_t) :                     8    sizeof(     uint64_t) :                     8
sizeof(int_least64_t) :                     8    sizeof(int_least64_t) :                     8
sizeof( int_fast64_t) :                     8    sizeof( int_fast64_t) :                     8
---------------------+-----------------------    ---------------------+-----------------------
             int64_vn :  -9223372036854775808                 int64_vn :  -9223372036854775808
             int64_vp :  +9223372036854775807                 int64_vp :  +9223372036854775807
            uint64_vp :  18446744073709551615                uint64_vp : +18446744073709551615
       int_least64_vn :  -9223372036854775808           int_least64_vn :  -9223372036854775808
       int_least64_vp :  +9223372036854775807           int_least64_vp :  +9223372036854775807
        int_fast64_vn :  -9223372036854775808            int_fast64_vn :  -9223372036854775808
        int_fast64_vp :  +9223372036854775807            int_fast64_vp :  +9223372036854775807
---------------------+-----------------------    ---------------------+-----------------------
PRId64: lld, PRIdLEAST64: lld, PRIdFAST64: lld   PRId64: lld, PRIdLEAST64: lld, PRIdFAST64: lld
PRIo64: llo, PRIoLEAST64: llo, PRIoFAST64: lld   PRIo64: llo, PRIoLEAST64: llo, PRIoFAST64: lld
PRIu64: llu, PRIuLEAST64: llu, PRIuFAST64: llu   PRIu64: llu, PRIuLEAST64: llu, PRIuFAST64: llu
PRIx64: llx, PRIxLEAST64: llx, PRIxFAST64: llx   PRIx64: llx, PRIxLEAST64: llx, PRIxFAST64: llx
PRIX64: llX, PRIXLEAST64: llX, PRIXFAST64: llX   PRIX64: llX, PRIXLEAST64: llX, PRIXFAST64: llX
=============================================    =============================================
Den Typ int96_t kennt dieser Compiler nicht!     Den Typ int96_t kennt dieser Compiler nicht!
=============================================    =============================================
Den Typ int128_t kennt dieser Compiler nicht!    Den Typ int128_t kennt dieser Compiler nicht!
=============================================    =============================================
IntTypes03 :  Das war's erstmal!                 IntTypes03 :  Das war's erstmal!
------------------------------------------------------------------------ */
