#define MAXBASE 1073741824
// ------------------------------Bignum representation---------------------
// The bignums are an opaque data type. The void * is a pointer to an array
// of 32 bit integers, but its representation is hidden so that any change
// in the bignum code will not affect users of the library, that without
// recompiling (since the bignums are a dll) can use the new library
typedef struct _mp {
	void *mp;
} pBignum;
// Note that this "structure" is just a glorified ***pointer***. You have
// to allocate space using quadnew, before you can use it!

extern int  ERNUM;   /* last recorded error       */
//initializing
// The precision is 30 bits precision for each 32 bit integers you have.
// The argument is the number of 32 bit integers you want for each number.
// To know the precision in bits you multiply by 30, i.e. if you set the
// precision to 10, you have 300 bits precision.

int BignumPrecision(int); // returns the old precision

// Creation of a bignum, initialized to the given integer
pBignum quadnew(int);
pBignum overloaded newBignum(void);
pBignum overloaded newBignum(int);
pBignum overloaded newBignum(double);
pBignum overloaded newBignum(char *);
pBignum overloaded newBignum(long long);
pBignum overloaded newBignum(long double);
// Conversion functions
// From ascii text
pBignum atoquad(unsigned char *inpbuf);
// From long
pBignum longToquad(int);
void long2quad(int,pBignum);
// From/to double
pBignum doubleToquad(double);
double quad2double(pBignum);
// from/to long long
long long quadToll(pBignum x);
pBignum longlongToquad(long long n);
// Copying
void quadcopy(pBignum destination,pBignum source);
// Formatting
void quadexpformat(pBignum x,unsigned char *outbuf,int width,int decimals);
void quadtoa(pBignum x,unsigned char *buffer);
void quadformat(pBignum x,unsigned char *buffer);

//----------------------------------------------------------------------
// Operators
//----------------------------------------------------------------------
// Comparisons
// Less than
int operator <(pBignum a,pBignum b);
int operator <(pBignum a,int b);
int operator <(pBignum a,double b);
int operator <(int a,pBignum b);
int operator <(double a,pBignum b);
// Less equal than
int operator <=(pBignum a,pBignum b);
int operator <=(pBignum a,double b);
int operator <=(int a,pBignum b);
int operator <=(double a,pBignum b);
// Equal
int operator ==(pBignum a,pBignum b);
int operator ==(pBignum a,int b);
int operator ==(pBignum a,double b);
int operator ==(int a,pBignum b);
int operator ==(double a,pBignum b);
// Greater equal
int operator >=(pBignum a,pBignum b);
int operator >=(pBignum a,int b);
int operator >=(pBignum a,double b);
int operator >=(int a,pBignum b);
int operator >=(double a,pBignum b);
// Greater than
int operator >(pBignum a,pBignum b);
int operator >(pBignum a,int b);
int operator >(pBignum a,double b);
int operator >(int a,pBignum b);
int operator >(double a,pBignum b);
// Different than
int operator !=(pBignum a,pBignum b);
int operator !=(pBignum a,int b);
int operator !=(pBignum a,double b);
int operator !=(int a,pBignum b);
int operator !=(double a,pBignum b);
// Returns -1 if a < b, 0 if a == b and 1 if a > b
int quadcmp(pBignum a,pBignum b);
// Basic math
// Addition
// This operators return a freshly allocated bignum
pBignum operator+(pBignum a,pBignum b);
pBignum operator+(pBignum a,int b);
pBignum operator+ (int a,pBignum b);
pBignum operator+(pBignum a,double b);
pBignum operator+(double a,pBignum b);
// This operators are more efficient and recycle their arguments.
// No new storage is allocated
pBignum operator+=(pBignum a,pBignum b);
pBignum operator+=(pBignum a,int b);
pBignum operator+=(pBignum a,double b);
pBignum operator++(pBignum a);
// Direct interface
void quadsum(pBignum a,pBignum b,pBignum result);
void quadincr(pBignum xx);
// unary minus
pBignum operator -(pBignum);
// Substraction
// This operators return a freshly allocated bignum
pBignum operator-(pBignum a,pBignum b);
pBignum operator-(pBignum a,int b);
pBignum operator- (int a,pBignum b);
pBignum operator-(pBignum a,double b);
pBignum operator-(double a,pBignum b);
// This operators are more efficient and recycle their arguments.
// No new storage is allocated
pBignum operator-=(pBignum a,int b);
pBignum operator-=(pBignum a,double b);
pBignum quaddecr(pBignum);
void quadsub(pBignum a,pBignum b,pBignum result);
// Multiplication
pBignum operator*(pBignum a,pBignum b);
pBignum operator*(int a,pBignum b);
pBignum operator*(pBignum a,int b);
pBignum operator*(double a,pBignum b);
pBignum operator*(pBignum a,double b);
pBignum operator*=(pBignum a,int b);
pBignum operator*=(pBignum a,double b);
void quadmult(pBignum a,pBignum b,pBignum result);
// Division
pBignum operator/(pBignum a,pBignum b);
pBignum operator/(int a,pBignum b);
pBignum operator/(pBignum a,int b);
pBignum operator/(pBignum a,double b);
pBignum operator/(double a,pBignum b);
pBignum operator/=(pBignum a,int b);
pBignum operator/=(pBignum a,double b);
void quaddiv(pBignum a,pBignum b,pBignum result);
pBignum quadreciprocal(pBignum);
// Modulous
pBignum quadmod(pBignum x,pBignum y);
pBignum operator%(pBignum x,pBignum y);

// direct operators
pBignum operator=(pBignum &,int);
pBignum operator=(pBignum &,double);
double operator=(double &,pBignum);
// Input output
#include <iostream.h>
iostream *operator <<(iostream *i,pBignum a);

//square root
pBignum quadisqrt(pBignum x);
pBignum quadsqrt(pBignum x);
int logb2(pBignum xx); //Logarithm base 2: number of bits in a bignum

// other operations
pBignum overloaded quadpow(pBignum,pBignum);
pBignum quadpowmod(pBignum xx,pBignum yy,pBignum zz);
pBignum overloaded quadpow(pBignum,int);
pBignum overloaded quadpow(int,int);
pBignum overloaded quadexp(pBignum);
pBignum overloaded quadexp(int);
pBignum quadrand(pBignum w);
pBignum quadln(pBignum);
// Trig
pBignum quadsin(pBignum);
pBignum quadcos(pBignum);
pBignum quadacos(pBignum);
pBignum quadasin(pBignum);
pBignum quadtan(pBignum);
pBignum quadatan(pBignum);
pBignum quadtanh(pBignum x);
pBignum quadatanh(pBignum x);
pBignum quadasinh(pBignum x);
pBignum quadsinh(pBignum x);
pBignum quadcosh(pBignum x);
pBignum quadacosh(pBignum x);
// pgcd
pBignum quadpgcd(pBignum xx,pBignum yy);
int quadintegerp(pBignum xx); // returns 1 if xx is an integer, zero otherwise
int quadcmp(pBignum a,pBignum b); // -1 if a < b 0 if a == b, and 1 if a > b
pBignum quadround(pBignum,int n);
// Return a bignum built from a ratio of two integers
pBignum quadratio(int numerator,int denominator);
pBignum fact(int n);
pBignum quadfloor(pBignum);
pBignum quadtrunc(pBignum);
pBignum quadratio(int n,int d);
#pragma lib <bignums.lib>
