#ifndef __d3dxmath_h__
#define __d3dxmath_h__
#include <d3d.h>
#include <math.h>
#include <limits.h>
#include "d3dxerr.h"
#ifndef D3DXINLINE
#define D3DXINLINE 
#endif
typedef struct ID3DXMatrixStack *LPD3DXMATRIXSTACK;
DEFINE_GUID( IID_ID3DXMatrixStack, 0xe3357330, 0xcc5e, 0x11d2, 0xa4, 0x34, 0x0, 0xa0, 0xc9, 0x6, 0x29, 0xa8);
#define D3DX_PI ((float) 3.141592654f)
#define D3DX_1BYPI ((float) 0.318309886f)
#define D3DXToRadian( degree ) ((degree) * (D3DX_PI / 180.0f))
#define D3DXToDegree( radian ) ((radian) * (180.0f / D3DX_PI))
typedef struct D3DXVECTOR2 { float x, y; } D3DXVECTOR2, *LPD3DXVECTOR2;
typedef struct D3DXVECTOR3 { float x, y, z; } D3DXVECTOR3, *LPD3DXVECTOR3;
typedef struct D3DXVECTOR4 { float x, y, z, w; } D3DXVECTOR4, *LPD3DXVECTOR4;
typedef struct D3DXMATRIX {
	union { float m[4][4]; };
} D3DXMATRIX, *LPD3DXMATRIX;
typedef struct D3DXQUATERNION { float x, y, z, w; } D3DXQUATERNION, *LPD3DXQUATERNION;
typedef struct D3DXPLANE { float a, b, c, d; } D3DXPLANE, *LPD3DXPLANE;
typedef struct D3DXCOLOR { FLOAT r, g, b, a; } D3DXCOLOR, *LPD3DXCOLOR;
float D3DXVec2Length ( const D3DXVECTOR2 *pV );
float D3DXVec2LengthSq ( const D3DXVECTOR2 *pV ); 
float D3DXVec2Dot ( const D3DXVECTOR2 *pV1, const D3DXVECTOR2 *pV2 ); 
float D3DXVec2CCW ( const D3DXVECTOR2 *pV1, const D3DXVECTOR2 *pV2 ); 
D3DXVECTOR2* D3DXVec2Add ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV1, const D3DXVECTOR2 *pV2 );
D3DXVECTOR2* D3DXVec2Subtract ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV1, const D3DXVECTOR2 *pV2 );
D3DXVECTOR2* D3DXVec2Minimize ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV1, const D3DXVECTOR2 *pV2 );
D3DXVECTOR2* D3DXVec2Maximize ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV1, const D3DXVECTOR2 *pV2 );
D3DXVECTOR2* D3DXVec2Scale ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV, float s );
D3DXVECTOR2* D3DXVec2Lerp ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV1, const D3DXVECTOR2 *pV2, float s );
D3DXVECTOR2* WINAPI D3DXVec2Normalize ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV );
D3DXVECTOR2* WINAPI D3DXVec2Hermite ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV1, const D3DXVECTOR2 *pT1, const D3DXVECTOR2 *pV2, const D3DXVECTOR2 *pT2, float s );
D3DXVECTOR2* WINAPI D3DXVec2BaryCentric ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV1, const D3DXVECTOR2 *pV2, D3DXVECTOR2 *pV3, float f, float g);
D3DXVECTOR4* WINAPI D3DXVec2Transform ( D3DXVECTOR4 *pOut, const D3DXVECTOR2 *pV, const D3DXMATRIX *pM );
D3DXVECTOR2* WINAPI D3DXVec2TransformCoord ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV, const D3DXMATRIX *pM );
D3DXVECTOR2* WINAPI D3DXVec2TransformNormal ( D3DXVECTOR2 *pOut, const D3DXVECTOR2 *pV, const D3DXMATRIX *pM );
float D3DXVec3Length ( const D3DXVECTOR3 *pV );
float D3DXVec3LengthSq ( const D3DXVECTOR3 *pV );
float D3DXVec3Dot ( const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2 );
D3DXVECTOR3* D3DXVec3Cross ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2 );
D3DXVECTOR3* D3DXVec3Add ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2 );
D3DXVECTOR3* D3DXVec3Subtract ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2 );
D3DXVECTOR3* D3DXVec3Minimize ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2 );
D3DXVECTOR3* D3DXVec3Maximize ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2 );
D3DXVECTOR3* D3DXVec3Scale ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV, float s);
D3DXVECTOR3* D3DXVec3Lerp ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2, float s );
D3DXVECTOR3* WINAPI D3DXVec3Normalize ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV );
D3DXVECTOR3* WINAPI D3DXVec3Hermite ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pT1, const D3DXVECTOR3 *pV2, const D3DXVECTOR3 *pT2, float s );
D3DXVECTOR3* WINAPI D3DXVec3BaryCentric ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2, const D3DXVECTOR3 *pV3, float f, float g);
D3DXVECTOR4* WINAPI D3DXVec3Transform ( D3DXVECTOR4 *pOut, const D3DXVECTOR3 *pV, const D3DXMATRIX *pM );
D3DXVECTOR3* WINAPI D3DXVec3TransformCoord ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV, const D3DXMATRIX *pM );
D3DXVECTOR3* WINAPI D3DXVec3TransformNormal ( D3DXVECTOR3 *pOut, const D3DXVECTOR3 *pV, const D3DXMATRIX *pM );
float D3DXVec4Length ( const D3DXVECTOR4 *pV );
float D3DXVec4LengthSq ( const D3DXVECTOR4 *pV );
float D3DXVec4Dot ( const D3DXVECTOR4 *pV1, const D3DXVECTOR4 *pV2 );
D3DXVECTOR4* D3DXVec4Add ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV1, const D3DXVECTOR4 *pV2);
D3DXVECTOR4* D3DXVec4Subtract ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV1, const D3DXVECTOR4 *pV2);
D3DXVECTOR4* D3DXVec4Minimize ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV1, const D3DXVECTOR4 *pV2);
D3DXVECTOR4* D3DXVec4Maximize ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV1, const D3DXVECTOR4 *pV2);
D3DXVECTOR4* D3DXVec4Scale ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV, float s); 
D3DXVECTOR4* D3DXVec4Lerp ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV1, const D3DXVECTOR4 *pV2, float s );
D3DXVECTOR4* WINAPI D3DXVec4Cross ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV1, const D3DXVECTOR4 *pV2, const D3DXVECTOR4 *pV3);
D3DXVECTOR4* WINAPI D3DXVec4Normalize ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV );
D3DXVECTOR4* WINAPI D3DXVec4Hermite ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV1, const D3DXVECTOR4 *pT1, const D3DXVECTOR4 *pV2, const D3DXVECTOR4 *pT2, float s ); 
D3DXVECTOR4* WINAPI D3DXVec4BaryCentric ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV1, const D3DXVECTOR4 *pV2, const D3DXVECTOR4 *pV3, float f, float g); 
D3DXVECTOR4* WINAPI D3DXVec4Transform ( D3DXVECTOR4 *pOut, const D3DXVECTOR4 *pV, const D3DXMATRIX *pM ); 
D3DXMATRIX* D3DXMatrixIdentity ( D3DXMATRIX *pOut );
BOOL D3DXMatrixIsIdentity ( const D3DXMATRIX *pM );
float WINAPI D3DXMatrixfDeterminant ( const D3DXMATRIX *pM );
D3DXMATRIX* WINAPI D3DXMatrixMultiply ( D3DXMATRIX *pOut, const D3DXMATRIX *pM1, const D3DXMATRIX *pM2 );
D3DXMATRIX* WINAPI D3DXMatrixTranspose ( D3DXMATRIX *pOut, const D3DXMATRIX *pM );
D3DXMATRIX* WINAPI D3DXMatrixInverse ( D3DXMATRIX *pOut, float *pfDeterminant, const D3DXMATRIX *pM );
D3DXMATRIX* WINAPI D3DXMatrixScaling ( D3DXMATRIX *pOut, float sx, float sy, float sz );
D3DXMATRIX* WINAPI D3DXMatrixTranslation ( D3DXMATRIX *pOut, float x, float y, float z );
D3DXMATRIX* WINAPI D3DXMatrixRotationX ( D3DXMATRIX *pOut, float angle );
D3DXMATRIX* WINAPI D3DXMatrixRotationY ( D3DXMATRIX *pOut, float angle );
D3DXMATRIX* WINAPI D3DXMatrixRotationZ ( D3DXMATRIX *pOut, float angle );
D3DXMATRIX* WINAPI D3DXMatrixRotationAxis ( D3DXMATRIX *pOut, const D3DXVECTOR3 *pV, float angle );
D3DXMATRIX* WINAPI D3DXMatrixRotationQuaternion ( D3DXMATRIX *pOut, const D3DXQUATERNION *pQ);
D3DXMATRIX* WINAPI D3DXMatrixRotationYawPitchRoll ( D3DXMATRIX *pOut, float yaw, float pitch, float roll );
D3DXMATRIX* WINAPI D3DXMatrixTransformation ( D3DXMATRIX *pOut, const D3DXVECTOR3 *pScalingCenter, const D3DXQUATERNION *pScalingRotation, const D3DXVECTOR3 *pScaling, const D3DXVECTOR3 *pRotationCenter, const D3DXQUATERNION *pRotation, const D3DXVECTOR3 *pTranslation); 
D3DXMATRIX* WINAPI D3DXMatrixAffineTransformation ( D3DXMATRIX *pOut, float Scaling, const D3DXVECTOR3 *pRotationCenter, const D3DXQUATERNION *pRotation, const D3DXVECTOR3 *pTranslation);
D3DXMATRIX* WINAPI D3DXMatrixLookAt ( D3DXMATRIX *pOut, const D3DXVECTOR3 *pEye, const D3DXVECTOR3 *pAt, const D3DXVECTOR3 *pUp ); 
D3DXMATRIX* WINAPI D3DXMatrixLookAtLH ( D3DXMATRIX *pOut, const D3DXVECTOR3 *pEye, const D3DXVECTOR3 *pAt, const D3DXVECTOR3 *pUp ); 
D3DXMATRIX* WINAPI D3DXMatrixPerspective ( D3DXMATRIX *pOut, float w, float h, float zn, float zf ); 
D3DXMATRIX* WINAPI D3DXMatrixPerspectiveLH ( D3DXMATRIX *pOut, float w, float h, float zn, float zf ); 
D3DXMATRIX* WINAPI D3DXMatrixPerspectiveFov ( D3DXMATRIX *pOut, float fovy, float aspect, float zn, float zf ); 
D3DXMATRIX* WINAPI D3DXMatrixPerspectiveFovLH ( D3DXMATRIX *pOut, float fovy, float aspect, float zn, float zf ); 
D3DXMATRIX* WINAPI D3DXMatrixPerspectiveOffCenter ( D3DXMATRIX *pOut, float l, float r, float b, float t, float zn, float zf );
D3DXMATRIX* WINAPI D3DXMatrixPerspectiveOffCenterLH ( D3DXMATRIX *pOut, float l, float r, float b, float t, float zn, float zf );
D3DXMATRIX* WINAPI D3DXMatrixOrtho ( D3DXMATRIX *pOut, float w, float h, float zn, float zf );
D3DXMATRIX* WINAPI D3DXMatrixOrthoLH ( D3DXMATRIX *pOut, float w, float h, float zn, float zf );
D3DXMATRIX* WINAPI D3DXMatrixOrthoOffCenter ( D3DXMATRIX *pOut, float l, float r, float b, float t, float zn, float zf ); 
D3DXMATRIX* WINAPI D3DXMatrixOrthoOffCenterLH ( D3DXMATRIX *pOut, float l, float r, float b, float t, float zn, float zf ); 
D3DXMATRIX* WINAPI D3DXMatrixShadow ( D3DXMATRIX *pOut, const D3DXVECTOR4 *pLight, const D3DXPLANE *pPlane ); 
D3DXMATRIX* WINAPI D3DXMatrixReflect ( D3DXMATRIX *pOut, const D3DXPLANE *pPlane );
float D3DXQuaternionLength ( const D3DXQUATERNION *pQ );
float D3DXQuaternionLengthSq ( const D3DXQUATERNION *pQ );
float D3DXQuaternionDot ( const D3DXQUATERNION *pQ1, const D3DXQUATERNION *pQ2 );
D3DXQUATERNION* D3DXQuaternionIdentity ( D3DXQUATERNION *pOut );
BOOL D3DXQuaternionIsIdentity ( const D3DXQUATERNION *pQ );
D3DXQUATERNION* D3DXQuaternionConjugate ( D3DXQUATERNION *pOut, const D3DXQUATERNION *pQ );
void WINAPI D3DXQuaternionToAxisAngle ( const D3DXQUATERNION *pQ, D3DXVECTOR3 *pAxis, float *pAngle ); 
D3DXQUATERNION* WINAPI D3DXQuaternionRotationMatrix ( D3DXQUATERNION *pOut, const D3DXMATRIX *pM); 
D3DXQUATERNION* WINAPI D3DXQuaternionRotationAxis ( D3DXQUATERNION *pOut, const D3DXVECTOR3 *pV, float angle ); 
D3DXQUATERNION* WINAPI D3DXQuaternionRotationYawPitchRoll ( D3DXQUATERNION *pOut, float yaw, float pitch, float roll );
D3DXQUATERNION* WINAPI D3DXQuaternionMultiply ( D3DXQUATERNION *pOut, const D3DXQUATERNION *pQ1, const D3DXQUATERNION *pQ2 ); 
D3DXQUATERNION* WINAPI D3DXQuaternionNormalize ( D3DXQUATERNION *pOut, const D3DXQUATERNION *pQ ); 
D3DXQUATERNION* WINAPI D3DXQuaternionInverse ( D3DXQUATERNION *pOut, const D3DXQUATERNION *pQ ); 
D3DXQUATERNION* WINAPI D3DXQuaternionLn ( D3DXQUATERNION *pOut, const D3DXQUATERNION *pQ );
D3DXQUATERNION* WINAPI D3DXQuaternionExp ( D3DXQUATERNION *pOut, const D3DXQUATERNION *pQ );
D3DXQUATERNION* WINAPI D3DXQuaternionSlerp ( D3DXQUATERNION *pOut, const D3DXQUATERNION *pQ1, const D3DXQUATERNION *pQ2, float t ); 
D3DXQUATERNION* WINAPI D3DXQuaternionSquad ( D3DXQUATERNION *pOut, const D3DXQUATERNION *pQ1, const D3DXQUATERNION *pQ2, const D3DXQUATERNION *pQ3, const D3DXQUATERNION *pQ4, float t ); 
D3DXQUATERNION* WINAPI D3DXQuaternionBaryCentric ( D3DXQUATERNION *pOut, const D3DXQUATERNION *pQ1, const D3DXQUATERNION *pQ2, const D3DXQUATERNION *pQ3, float f, float g ); 
float D3DXPlaneDot ( const D3DXPLANE *pP, const D3DXVECTOR4 *pV); 
float D3DXPlaneDotCoord ( const D3DXPLANE *pP, const D3DXVECTOR3 *pV); 
float D3DXPlaneDotNormal ( const D3DXPLANE *pP, const D3DXVECTOR3 *pV); 
D3DXPLANE* WINAPI D3DXPlaneNormalize ( D3DXPLANE *pOut, const D3DXPLANE *pP); 
D3DXVECTOR3* WINAPI D3DXPlaneIntersectLine ( D3DXVECTOR3 *pOut, const D3DXPLANE *pP, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2); 
D3DXPLANE* WINAPI D3DXPlaneFromPointNormal ( D3DXPLANE *pOut, const D3DXVECTOR3 *pPoint, const D3DXVECTOR3 *pNormal); 
D3DXPLANE* WINAPI D3DXPlaneFromPoints ( D3DXPLANE *pOut, const D3DXVECTOR3 *pV1, const D3DXVECTOR3 *pV2, const D3DXVECTOR3 *pV3); 
D3DXPLANE* WINAPI D3DXPlaneTransform ( D3DXPLANE *pOut, const D3DXPLANE *pP, const D3DXMATRIX *pM ); 
D3DXCOLOR* D3DXColorNegative (D3DXCOLOR *pOut, const D3DXCOLOR *pC); 
D3DXCOLOR* D3DXColorAdd (D3DXCOLOR *pOut, const D3DXCOLOR *pC1, const D3DXCOLOR *pC2); 
D3DXCOLOR* D3DXColorSubtract (D3DXCOLOR *pOut, const D3DXCOLOR *pC1, const D3DXCOLOR *pC2); 
D3DXCOLOR* D3DXColorScale (D3DXCOLOR *pOut, const D3DXCOLOR *pC, float s); 
D3DXCOLOR* D3DXColorModulate (D3DXCOLOR *pOut, const D3DXCOLOR *pC1, const D3DXCOLOR *pC2); 
D3DXCOLOR* D3DXColorLerp (D3DXCOLOR *pOut, const D3DXCOLOR *pC1, const D3DXCOLOR *pC2, float s);
D3DXCOLOR* WINAPI D3DXColorAdjustSaturation (D3DXCOLOR *pOut, const D3DXCOLOR *pC, float s);
D3DXCOLOR* WINAPI D3DXColorAdjustContrast (D3DXCOLOR *pOut, const D3DXCOLOR *pC, float c);
DECLARE_INTERFACE_(ID3DXMatrixStack, IUnknown)
{
	STDMETHOD(QueryInterface)(THIS_ REFIID riid, LPVOID * ppvObj) PURE;
	STDMETHOD_(ULONG,AddRef)(THIS) PURE;
	STDMETHOD_(ULONG,Release)(THIS) PURE;
	STDMETHOD(Pop)(THIS) PURE;
	STDMETHOD(Push)(THIS) PURE;
	STDMETHOD(LoadIdentity)(THIS) PURE;
	STDMETHOD(LoadMatrix)(THIS_ const D3DXMATRIX* pM ) PURE;
	STDMETHOD(MultMatrix)(THIS_ const D3DXMATRIX* pM ) PURE;
	STDMETHOD(MultMatrixLocal)(THIS_ const D3DXMATRIX* pM ) PURE;
	STDMETHOD(RotateAxis)
	(THIS_ const D3DXVECTOR3* pV, float angle) PURE;
	STDMETHOD(RotateAxisLocal)
	(THIS_ const D3DXVECTOR3* pV, float angle) PURE;
	STDMETHOD(RotateYawPitchRoll)
	(THIS_ float yaw, float pitch, float roll) PURE;
	STDMETHOD(RotateYawPitchRollLocal)
	(THIS_ float yaw, float pitch, float roll) PURE;
	STDMETHOD(Scale)(THIS_ float x, float y, float z) PURE;
	STDMETHOD(ScaleLocal)(THIS_ float x, float y, float z) PURE;
	STDMETHOD(Translate)(THIS_ float x, float y, float z ) PURE;
	STDMETHOD(TranslateLocal)(THIS_ float x, float y, float z) PURE;
	STDMETHOD_(D3DXMATRIX*, GetTop)(THIS) PURE;
};
HRESULT WINAPI D3DXCreateMatrixStack( DWORD flags, LPD3DXMATRIXSTACK *ppStack );
#include "d3dxmath.inl"
#endif 
