// Definition of the int128 data type for 32 bit machines
typedef struct _int128 {
	unsigned lolo;
	unsigned lohi;
	unsigned hilo;
	int hihi;
} int128;

#define INT128_ERROR_OVERFLOW	1
#define INT128_ERROR_ZERODIV	2

int i128Mod(int128 *result,int128 left,int128 right);
int i128Div(int128 *result,int128 left,int128 right);
int i128Idiv(int128 *result,int128 left,int128 right);
int i128Mul(int128 *result,int128 left,int128 right);
int i128Mulbyint(int128 *result,int128 *src1,int src2);
int  i128Abs(int128 *result,int128 *src);
void i128Neg(int128 *result,int128 *src);
int i128Sub(int128 *result,int128 *src1,int128 *src2);
int i128Add(int128 *result,int128 *src1,int128 *src2);
int i128Addint(int128 *result,int128 *src,int src2);
void i128Or(int128 *result,int128 *src1,int128 *src2);
void i128Xor(int128 *result,int128 *src1,int128 *src2);
void i128And(int128 *result,int128 *src1,int128 *src2);
void i128Not(int128 *result,int128 *src);
void i128asgn(int128 *lhs,int128 *rhs);
// Note that the shift argument is NOT a pointer
void i128LShift(int128 *result,int128 src,int shiftAmount);
void i128RShift(int128 *result,int128 src,int shiftAmount);
int  i128Toint(int128 *d);
void int2i128(int128 *a,int b);
void uint2i128(int128 *a,unsigned b);
void i128toa(int128 *in,char *out);
int  i128IsZero(int128 *in);
int  i128Cmp(int128 *a,int128 *b);
int  i128Equalint(int128 *,int);
int  i128Equal(int128 *,int128 *);
int  i128Less(int128 *,int128 *);
int  i128incr(int128 *result,int128 *src);
void longlong2i128(int128 *result,__int64 data); 
__int64 i128tolonglong(int128 *data);
long double i128Tolongdouble(int128 *);
double i128Todouble(int128 *);
int longdoubleToi128(int128 *result,long double s);
int doubleToi128(int128 *result,double s);
int strtoi128(int128 *result,char *srcStr,char **pendStr,int base);
#ifdef __LCC__
//
// operators
//
int128 operator+(int128 &src1,int128 & src2);
int128 operator+(int128 &src1,int src2);

int128 operator-(int128 &src1,int128 &src2);
int128 operator-(int128 &src1,int src2);

int128 operator*(int128 left,int128 right);
int128 operator/(int128 divisor,int128 dividend);
int128 operator&(int128 &src1,int128 &src2);
int128 operator|(int128 &src1,int128 &src2);
int128 operator^(int128 &,int128 &);
int  operator!(int128 &src);
// Shift operators
int128 operator<<(int128 ,int);
int128 operator>>(int128,int);
// Assignment operator
int128 operator=(int128 &,int);
int128 operator=(int128 &,unsigned);
int128 operator=(int128 &,long long);
int128 operator=(int128 &,unsigned long long);
// Casts
int128 operator()(int);
int128 operator()(long long);
int128 operator++(int128 &);
int128 operator--(int128 &);
int  operator()(int128 &);
long long operator()(int128 &);
long double operator()(int128 &);
double operator()(int128 &);
//Comparisons
int operator< (int128 &left,int128 &right);
int operator<=(int128 &left,int right);
int operator==(int128 &left,int128 &right);
int operator==(int128 &left,int right);

#define INT128_C(a) (a##I128)
#endif
