#ifndef __stdio_h__
#define __stdio_h__

#ifndef _SIZE_T_DEFINED
typedef unsigned int size_t;
#define _SIZE_T_DEFINED
#endif

#ifndef _WCHAR_T_DEFINED
typedef unsigned short wchar_t;
#define _WCHAR_T_DEFINED
#endif

#ifndef _WCTYPE_T_DEFINED
typedef wchar_t wint_t;
typedef wchar_t wctype_t;
#define _WCTYPE_T_DEFINED
#endif

#ifndef _VA_LIST_DEFINED
typedef char *	va_list;
#define _VA_LIST_DEFINED
#endif

#define BUFSIZ	512

#define EOF	(-1)

struct _iobuf {
	char *_ptr;
	int   _cnt;
	char *_base;
	int   _flag;
	int   _file;
	int   _charbuf;
	int   _bufsiz;
	char *_tmpfname;
	};
typedef struct _iobuf FILE;


#ifndef __ANSIC__ONLY__
/* Directory where temporary files may be created. */
#define _P_tmpdir   "\\"
#define _wP_tmpdir  L"\\"
#define L_tmpnam (sizeof(_P_tmpdir)+12)
#else
#define L_tmpnam 13
#endif

/* Seek method constants */

#define SEEK_CUR    1
#define SEEK_END    2
#define SEEK_SET    0


#define FILENAME_MAX	260
#define FOPEN_MAX	20
#define TMP_MAX 	32767

/* Define NULL pointer value */
#ifndef NULL
#define NULL	((void *)0)
#endif

/* Define file position type */

#ifndef _FPOS_T_DEFINED
typedef long fpos_t;
#define _FPOS_T_DEFINED
#endif

extern FILE (*_imp___iob)[];
#define _iob    (*_imp___iob)
#define stdin  (&_iob[0])
#define stdout (&_iob[1])
#define stderr (&_iob[2])

#define _IOREAD 	0x0001
#define _IOWRT		0x0002

#define _IOFBF		0x0000
#define _IOLBF		0x0040
#define _IONBF		0x0004

#ifndef __ANSIC__ONLY__
#define _IOMYBUF	0x0008
#define _IOEOF		0x0010
#define _IOERR		0x0020
#define _IOSTRG 	0x0040
#define _IORW		0x0080
#define	_IOAPPEND	0x0200
#endif
/*
 * Function prototypes 
 */
void clearerr(FILE *);
int fclose(FILE *);

int feof(FILE *);
int ferror(FILE *);
int fflush(FILE *);
int fgetc(FILE *);
wchar_t fgetwc(FILE *);
wchar_t getwc(FILE *);
#define getwc fgetwc
int _fgetchar(void);
int fgetpos(FILE *, fpos_t *);
char * fgets(char *, int, FILE *);

FILE * fopen(const char *, const char *);
FILE * wfopen(const wchar_t *,const wchar_t *);
int fprintf(FILE *, const char *, ...);

int fputc(int, FILE *);
int _fputchar(int);
int fputs(const char *, FILE *);
size_t fread(void *, size_t, size_t, FILE *);
FILE * freopen(const char *, const char *, FILE *);
int fscanf(FILE *, const char *, ...);
int fsetpos(FILE *, const fpos_t *);
int fseek(FILE *, long, int);
long ftell(FILE *);
size_t fwrite(const void *, size_t, size_t, FILE *);
int getc(FILE *);
int getchar(void);
char * gets(char *);
int getw(FILE *);
int printf(const char *, ...);
int putc(int, FILE *);
int putchar(int);
int puts(const char *);
int remove(const char *);
int rename(const char *,const char *);
void rewind(FILE *);
int scanf(const char *, ...);
void setbuf(FILE *, char *);
int setvbuf(FILE *, char *, int, size_t);
int sprintf(char *, const char *, ...);
int snprintf(char *,size_t,const char *,...);
int sscanf(const char *, const char *, ...);
int vfscanf(FILE * restrict stream, const char * restrict format, va_list arg);
int vscanf(const char * restrict format, va_list arg);
FILE * tmpfile(void);
char * tmpnam(char *);
char *tempnam(char *,char *);
int ungetc(int, FILE *);
int vfprintf(FILE *, const char *, va_list);
int vprintf(const char *, va_list);
int vsnprintf(char *,size_t,const char *,va_list);
int vsprintf(char *, const char *, va_list);
void perror(char *);
void _wperror(const wchar_t *);

#ifndef __ANSIC__ONLY__
#include "safelib.h"
errno_t tmpfile_s(FILE * restrict * restrict streamptr);
errno_t tmpnam_s(char *s, rsize_t maxsize);
errno_t fopen_s(FILE * restrict * restrict streamptr, const char * restrict filename, const char * restrict mode);
errno_t freopen_s(FILE * restrict * restrict newS, const char * restrict fname, const char * restrict m, FILE * restrict s);
int fprintf_s(FILE * restrict stream, const char * restrict format, ...);
int fscanf_s(FILE * restrict stream, const char * restrict format, ...);
int printf_s(const char * restrict format, ...);
int scanf_s(const char * restrict format, ...);
int snprintf_s(char * restrict s, rsize_t n, const char * restrict format, ...);
int sprintf_s(char * restrict s, rsize_t n, const char * restrict format, ...);
int sscanf_s(const char * restrict s, const char * restrict format, ...);
int vfprintf_s(FILE * restrict stream, const char * restrict format, va_list arg);
int vfscanf_s(FILE * restrict stream, const char * restrict format, va_list arg);
int vprintf_s(const char * restrict format, va_list arg);
int vscanf_s(const char * restrict format, va_list arg);
int vsnprintf_s(char * restrict s, rsize_t n, const char * restrict format, va_list arg);
int vsprintf_s(char * restrict s, rsize_t n, const char * restrict format, va_list arg);
int vsscanf_s(const char * restrict s, const char * restrict format, va_list arg);
char *gets_s(char *s, rsize_t n);

#ifndef MAX_PATH
#define MAX_PATH	260
#endif
#define TMP_MAX	32767
#ifndef MAX_FNAME
#define MAX_FNAME	256
#endif
int dprintf(const char *, ...);
int getline(char **LinePointer,int *n,FILE *stream);
int getdelim(char **LinePointer,int *n, int delimiter, FILE *stream);
int fggets(char* *ln, FILE *f);
#define ggets(ln) fggets(ln, stdin)
// This is Paul Hsieh's code
long fgetstr (char * buff, int n, FILE * fp);
size_t fgetstralloc (char ** p, FILE * fp);
int _filbuf(FILE *);
int flsbuf(int, FILE *);
int asprintf(char **result, const char *format,...);
int vasprintf(char **result, const char *format,va_list args);
FILE * _fsopen(const char *, const char *, int);
int _fcloseall(void);
FILE * fdopen(int, const char *);
int fileno(FILE *);
#define _fileno fileno
int _flushall(void);
int _pclose(FILE *);
#define pclose _pclose
FILE * popen(const char *, const char *);
int _putw(int, FILE *);
int _vsnprintf(char *, size_t, const char *, va_list);
int _vsnwprintf(wchar_t *, size_t, const wchar_t *, va_list);
int xvsnprintf(char *,size_t,const char *,va_list);
int xvsnprintf(char *,size_t,const char *,va_list);
int xvsprintf(char *, const char *, va_list);
int _unlink(const char *);
int _snprintf(char *, size_t, const char *, ...);
char * _tempnam(char *, char *);

long long fseeki64(FILE *f,long long offset,int origin);
long long ftelli64(FILE *fhandle);
int _rmtmp(void);
#define unlink _unlink
#include <_syslist.h>
#ifndef STD_INPUT_HANDLE
#define STD_INPUT_HANDLE   ((unsigned long)-10)
#define STD_OUTPUT_HANDLE ((unsigned long)-11)
#define STD_ERROR_HANDLE  ((unsigned long)-12)
#endif
#endif
#endif
