#ifndef __wctype_h__
#define __wctype_h__
#pragma pack(push,8)

#ifndef _WCHAR_T_DEFINED
typedef unsigned short wchar_t;
#define _WCHAR_T_DEFINED
#endif

#ifndef _WCTYPE_T_DEFINED
typedef wchar_t wint_t;
typedef wchar_t wctype_t;
#define _WCTYPE_T_DEFINED
#endif


#ifndef WEOF
#define WEOF (wint_t)(0xFFFF)
#endif

/*
 * This declaration allows the user access to the ctype look-up
 * array _ctype defined in ctype.obj by simply including ctype.h
 */

#ifdef __LCCLIBC__
extern unsigned char __declspec(dllimport) _ctype[];
#else
extern  unsigned char   _ctype[];
#endif

extern unsigned short *_pctype;
extern wctype_t *_pwctype;


/* set bit masks for the possible character types */

#define _UPPER          1     /* upper case letter */
#define _LOWER          2     /* lower case letter */
#define _DIGIT          4     /* digit[0-9] */
#define _SPACE          8     /* tab, carriage return, newline, */
                                /* vertical tab or form feed */
#define _PUNCT          16    /* punctuation character */
#define _CONTROL        32    /* control character */
#define _BLANK          0x40    /* space char */
#define _HEX            0x80    /* hexadecimal digit */

#define _LEADBYTE       0x8000                  /* multibyte leadbyte */
#define _ALPHA          (0x0100|_UPPER|_LOWER)  /* alphabetic character */

#ifndef _WCTYPE_DEFINED

/* Character classification function prototypes */
/* also declared in ctype.h */

int iswalpha(wint_t);
int iswupper(wint_t);
int iswlower(wint_t);
int iswdigit(wint_t);
int iswxdigit(wint_t);
int iswspace(wint_t);
int iswpunct(wint_t);
int iswalnum(wint_t);
int iswprint(wint_t);
int iswgraph(wint_t);
int iswcntrl(wint_t);
int iswascii(wint_t);
int isleadbyte(int);
wchar_t towupper(wchar_t);
wchar_t towlower(wchar_t);
int iswctype(wint_t, wctype_t);
#define _WCTYPE_DEFINED
#endif

#define iswalpha(_c)    ( iswctype(_c,_ALPHA) )
#define iswupper(_c)    ( iswctype(_c,_UPPER) )
#define iswlower(_c)    ( iswctype(_c,_LOWER) )
#define iswdigit(_c)    ( iswctype(_c,_DIGIT) )
#define iswxdigit(_c)   ( iswctype(_c,_HEX) )
#define iswspace(_c)    ( iswctype(_c,_SPACE) )
#define iswpunct(_c)    ( iswctype(_c,_PUNCT) )
#define iswalnum(_c)    ( iswctype(_c,_ALPHA|_DIGIT) )
#define iswprint(_c)    ( iswctype(_c,_BLANK|_PUNCT|_ALPHA|_DIGIT) )
#define iswgraph(_c)    ( iswctype(_c,_PUNCT|_ALPHA|_DIGIT) )
#define iswcntrl(_c)    ( iswctype(_c,_CONTROL) )
#define iswascii(_c)    ( (unsigned)(_c) < 0x80 )
#define isleadbyte(_c)  (_pctype[(unsigned char)(_c)] & _LEADBYTE)

typedef wchar_t wctrans_t;
wint_t towctrans(wint_t, wctrans_t);
wctrans_t wctrans(const char *);
wctype_t wctype(const char *);
#pragma pack(pop)
#endif
