/*------------------------------------------------------------------------
 Procedure:     ResizeToolbar ID:1
 Purpose:       Tries to resize the toolbar according to the new 
                width, adding lines to the toolbar if necessary.
 Input:         hwnd: The parent window, i.e. the floating toolbar 
                window
                nButtons: the number of buttons in the toolbar
                lParam: the lParam parameter of the WM_SIZE message
                bwidth: the width ofeach button
 Output:        The floating toolbar window is resized
 Errors:        
------------------------------------------------------------------------*/
int ResizeToolbar(HWND hwnd,int nButtons,long lParam,int bwidth)
{
        RECT rcb;
		int n,old,result,width,height;
		HWND hwndParent;
		DWORD style;

        if (hwnd == (HWND)0) return 0;
        memset(&rcb,0,sizeof(RECT));
        old = (int)SendMessage(hwnd,TB_GETROWS,0,0);
        if (bwidth > 0)
                n = LOWORD(lParam) / (bwidth);
        else return 0;
        if (n < nButtons) {
			n = 1 + nButtons/n;
        }
        else n = 1;
        SendMessage(hwnd,TB_SETROWS,MAKEWPARAM(n,TRUE),(LPARAM)&rcb);
        SendMessage(hwnd,TB_AUTOSIZE,0,0);
		hwndParent = GetParent(hwnd);
		style = GetWindowLong(hwndParent,GWL_STYLE);
		AdjustWindowRect(&rcb,style,0);
		MapWindowPoints(hwndParent,HWND_DESKTOP,(LPPOINT)&rcb,2);
		width = nButtons/(n);
		if (n >1 )
			width += nButtons & 1;
		width *= bwidth;
		width += GetSystemMetrics(SM_CXDLGFRAME);
		width += GetSystemMetrics(SM_CXBORDER);
		height = rcb.bottom - rcb.top;
		MoveWindow(hwndParent,rcb.left,rcb.top,width,height,1);
        result = 1;
        return(result);
}
/*------------------------------------------------------------------------
 Procedure:     FloatingToolbarProc ID:1
 Purpose:       Main window procedure for the floating toolbar.
                Messages handled:
                WM_COMMAND: send the message to hwndMain
                WM_SIZE: Resizes the floating toolbar
                WM_USER+100: Receives the handle of the toolbar 
                window.
                WM_CLOSE: Hides the window istead of really closing 
                it. You can show it again with ShowWindow

 Input:         Standard callback
 Output:        Depends on the message
 Errors:        
------------------------------------------------------------------------*/
LRESULT CALLBACK FloatingToolbarProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
	RECT rc;
	static HWND hwndTB;
	static int buttonw,buttonCount;
	static int maxwidth;
	static int isInSizeMsg;

	switch(msg) {
		case WM_CREATE:
			isInSizeMsg = 0;
		return 1;
		case WM_COMMAND:
		SendMessage(hwndMain,msg,wParam,lParam);
		break;
		case WM_USER+100:
			hwndTB = (HWND)lParam;
			SendMessage(hwndTB,TB_GETITEMRECT,0,(LPARAM)&rc);
			buttonw = rc.right;
			buttonCount = SendMessage(hwndTB,TB_BUTTONCOUNT,0,0);
			maxwidth = buttonCount * rc.right;
			break;
		case WM_SIZE:
			if (isInSizeMsg)
				return 0;
			isInSizeMsg = 1;
			if (hwndTB)
				ResizeToolbar(hwndTB,buttonCount,lParam,buttonw);
			SetFocus(hwndTB);
			isInSizeMsg = 0;
			return 0;
		case WM_CLOSE:
			ShowWindow(hwnd,SW_HIDE);
			return 0;

	}
	return DefWindowProc(hwnd,msg,wParam,lParam);
}
/*------------------------------------------------------------------------
 Procedure:     CreateFloatingToolbarWindow ID:1
 Purpose:       Creates the window that will enclose the tool bar. 
                This window will receive the messages of the toolbar, 
                and will pass the WM_COMMAND messages to the main 
                window.
 Input:         
 Output:        
 Errors:        
------------------------------------------------------------------------*/
HWND CreateFloatingToolbarWindow(HWND hwndToolBar)
{
	return CreateWindow("FloatingToolbar","Appl toolbar",
		WS_POPUP|WS_VISIBLE|WS_CAPTION|WS_BORDER|WS_THICKFRAME|WS_SYSMENU,
		0,0,0,0,
		hwndMain,
		NULL,
		hInst,
		hwndToolBar);
}

static void AdjustFloatingToolbarWindow(HWND hwndft,HWND hwndTB)
{
	RECT rc,rc1;
	DWORD Style;
	int height,nbuttons,width;
	POINT pt;

	SendMessage(hwndTB,TB_GETITEMRECT,0,(LPARAM)&rc);
	nbuttons = SendMessage(hwndTB,TB_BUTTONCOUNT,0,0);
	GetWindowRect(hwndft,&rc1);
	height = rc.bottom+GetSystemMetrics(SM_CYCAPTION)+
		GetSystemMetrics(SM_CYFRAME)+2*GetSystemMetrics(SM_CYDLGFRAME);
	Style = GetWindowLong(hwndft,GWL_STYLE);
	width = (nbuttons+1) * rc.right;
	width += 2*GetSystemMetrics(SM_CXDLGFRAME);
	SendMessage(hwndft,WM_USER+100,0,(LPARAM)hwndTB);
	pt.x = pt.y = 0;
	MapWindowPoints(hwndMain,HWND_DESKTOP,(LPPOINT)&pt,1);
	MoveWindow(hwndft,pt.x,pt.y,width,height,1);
}

/*------------------------------------------------------------------------
 Procedure:     CreateAToolBar ID:1
 Purpose:       Handles toolbar creation for the floating tool bar.
 Input:         The parent window, i.e. hwndMain. this parameter is 
                reused later for different purposes, since the parent 
                window of the toolbar is no longuer hwndMain, but the 
                floating toolbar window
 Output:        
 Errors:        
------------------------------------------------------------------------*/
static HWND CreateAToolBar(HWND hwndParent) 
{ 
	HWND hwndTB; 
	TBADDBITMAP tbab; 
	TBBUTTON tbb[NUM_TOOLBAR_BUTTONS]; 
	int index; 
 
	// Ensure that the common control DLL is loaded. 
	InitCommonControls(); 
 
	// Create a toolbar that the user can customize and that has a 
	// tooltip associated with it. 
	hwndParent = CreateFloatingToolbarWindow(hwndParent);
	hwndTB = CreateWindowEx(0, TOOLBARCLASSNAME, (LPSTR) NULL, 
	    /*@@TOOLBARSTYLE@@*/, 
	    0, 0, 0, 0, hwndParent, (HMENU) ID_TOOLBAR, hInst, NULL); 
 
	// Send the TB_BUTTONSTRUCTSIZE message, which is required for 
	// backward compatibility. 
	SendMessage(hwndTB, TB_BUTTONSTRUCTSIZE, (WPARAM) sizeof(TBBUTTON), 0); 
 
	// Add the bitmap containing button images to the toolbar. 
	tbab.hInst = HINST_COMMCTRL; 
	tbab.nID   = /*@@LARGE/SMALLSIZE@@*/; 
	SendMessage(hwndTB, TB_ADDBITMAP, (WPARAM) NUM_TOOLBAR_BUTTONS,(LPARAM) &tbab); 

	// clean memory before using it
	memset(tbb,0,sizeof tbb);
 
