#include <complex.h>
#define PRECL 32
#define MAXEXPL	16384
#define MINEXPL	-16384
long double cabsl(long double _Complex z )
{
	long double x, y, b, re, im;
	int ex, ey, e;

	x = creal(z);
	y = cimag(z);

/* Note, cabs(INFINITY,NAN) = INFINITY. */
	if(isinfl(x) || isinfl(y))
	return( infinity() );

	if( isnanl(x) )
		return(x);
	if( isnanl(y) )
		return(y);

	re = fabsl( x );
	im = fabsl( y );

	if( re == 0.0L )
		return( im );
	if( im == 0.0L )
		return( re );

	/* Get the exponents of the numbers */
	x = frexpl( re, &ex );
	y = frexpl( im, &ey );

	/* Check if one number is tiny compared to the other */
	e = ex - ey;
	if( e > PRECL )
		return( re );
	if( e < -PRECL )
		return( im );

	/* Find approximate exponent e of the geometric mean. */
	e = (ex + ey) >> 1;

	/* Rescale so mean is about 1 */
	x = ldexpl( re, -e );
	y = ldexpl( im, -e );
		
	/* Hypotenuse of the right triangle */
	b = sqrtl( x * x  +  y * y );

	/* Compute the exponent of the answer. */
	y = frexpl( b, &ey );
	ey = e + ey;

	/* Check it for overflow and underflow. */
	if( ey > MAXEXPL )
	{
		return( infinity() );
	}
	if( ey < MINEXPL )
		return(0.0L);

	/* Undo the scaling */
	b = ldexpl( b, e );
	return( b );
}
double cabs(long double _Complex x)
{
	return (double)cabsl(x);
}
