/*------------------------------------------------------------------------
Module:        d:\lcc\complex\COMPLEX.C
Author:        Jacob Navia
Project:       lcc-win32
State:
Creation Date: June 2000
Description:   This implements complex arithmetic.
Most operators are implemented with
reference arguments for efficiency. As
soon as "inline" is there, they will be
most inlined
------------------------------------------------------------------------*/

#include <complex.h>
float _Complex I = 0.0+1.0i;
// Comparisons
int EXPORT operator == (const double _Complex x, const double _Complex y)
{
	return x.re == y.re && x.im == y.im;
}

int EXPORT operator == (const double _Complex x, double y)
{
	return x.re == y && x.im == 0.0;
}

int EXPORT operator == (double x, const double _Complex y)
{
	return x == y.re && y.im == 0;
}

int EXPORT operator != (const double _Complex x, const double _Complex y)
{
	return x.re != y.re || x.im != y.im;
}
int EXPORT operator !=(const long double _Complex x, const double y)
{
	return x.re != y || x.im != 0.0;
}

int EXPORT operator != (double x, const double _Complex y)
{
	return x != y.re || y.im != 0.0;
}
//
// Assignments
//
long EXPORT double _Complex operator=(long double _Complex &a,long double d)
{
	a.re = d;
	a.im = 0;
	return a;
}

double _Complex EXPORT operator=(double _Complex &a,double d)
{
	a.re = d;
	a.im = 0;
	return a;
}

long double _Complex EXPORT operator=(long double _Complex &a,float _Complex d)
{
	a.re = d.re;
	a.im = d.im;
	return a;
}



double norm (const double _Complex x)
{
	return x.re * x.re + x.im * x.im;
}

int operator != (double _Complex x,int i)
{
	return x.re != (double)i || x.im != 0.0;
}

////////////////////////
// Comparison
////////////////////////

int operator != (int i,long double _Complex x)
{
        return x.re != (long double)i || x.im != 0.0;
}

double carg(const double _Complex x)
{
	return atan2(x.im, x.re);
}

long double cargl(const long double _Complex x)
{
	return atan2l(x.im,x.re);
}

double _Complex c_polar(double r, double t)
{
	double _Complex result;

	result.re = r * cos(t);
	result.im = r * sin(t);
	return result;
}

long double _Complex c_polarl(long double r, long double t)
{
	long double _Complex result;
	result.re = r*cosl(t);
	result.im = r*sinl(t);
	return result;
}

float _Complex c_polarf(float r,float t)
{
	float _Complex result;
	result.re = r*cosf(t);
	result.im = r*sinf(t);
	return result;
}

long double _Complex operator ~(long double _Complex c)
{
	long double _Complex result;

	result.re = ~(int)c.re;
	result.im=0;
	return result;
}

