#include <complex.h>
#undef creal
#undef cimag
#define creal(x) (x.re)
#define cimag(x) (x.im)

double _Complex c_pow_double(const double _Complex x, double y)
{
	return cexp(y * clog(x));
}

double _Complex c_double_pow(double x, const double _Complex y)
{
	return cexp(y * log (x));
}
/* calculate cosh and sinh */

static void cchshl(long double x,long double *c,long double *s )
{
	long double e, ei;

	if( fabs(x) <= 0.5L ) {
        	*c = coshl(x);
        	*s = sinhl(x);
        }
	else {
        	e = expl(x);
        	ei = 0.5L/e;
        	e = 0.5L * e;
        	*s = e - ei;
        	*c = e + ei;
        }
}
static double _stdcall redupi(double x);
static long double ctans (long double _Complex z)
{
  long double f, x, x2, y, y2, rn, t;
  long double d;

  x = fabs(2.0 * creal (z));
  y = fabs(2.0 * cimag(z));

  x = redupi(x);

  x = x * x;
  y = y * y;
  x2 = 1.0;
  y2 = 1.0;
  f = 1.0;
  rn = 0.0;
  d = 0.0;
  do
    {
      rn += 1.0;
      f *= rn;
      rn += 1.0;
      f *= rn;
      x2 *= x;
      y2 *= y;
      t = y2 + x2;
      t /= f;
      d += t;

      rn += 1.0;
      f *= rn;
      rn += 1.0;
      f *= rn;
      x2 *= x;
      y2 *= y;
      t = y2 - x2;
      t /= f;
      d += t;
    }
  while (fabs(t/d) > 1.084202172485504434007452e-19L);
  return (d);
}

long double _Complex EXPORT csinl(long double _Complex z)
{
  long double _Complex w;
  long double ch, sh;

  cchshl( cimag (z), &ch, &sh );
  w = sinl(creal(z)) * ch + (cosl(creal(z)) * sh) * I;
  return (w);
}

long double _Complex EXPORT csinhl(const long double _Complex x)
{
        long double _Complex result;

        result.re = sinhl(x.re) * cosl(x.im);
        result.im = coshl(x.re) * sinl(x.im);
        return result;
}

long double _Complex EXPORT  ccosl(long double _Complex z)
{
  long double _Complex w;
  long double ch, sh;

  cchshl( cimag(z), &ch, &sh );
  w = cosl(creal (z)) * ch - (sinl(creal (z)) * sh) * I;
  return (w);
}

long double _Complex __declspec(naked) ctanl(long double _Complex z)
{
}
float _Complex __declspec(naked) ctanf(float _Complex z)
{
}
void __ctanl(long double _Complex &w,long double _Complex z)
{
  long double d;

  d = cosl(2.0 * creal (z)) + coshl(2.0 * cimag (z));

  if (fabs(d) < 0.25)
    d = ctans(z);

  if (d == 0.0 )
    {
      w.re = infinity();
      w.im = infinity();
      return;
    }

  w = sinl(2.0 * creal(z)) / d + (sinhl(2.0 * cimag(z)) / d) * I;
}

double _Complex c_intpow (const double _Complex xin, int y)
{
	double _Complex result;
	if (y == 0) {
		result.re = 1.0;
		result.im = 0.0;
		return result;
	}
	double _Complex r =
		1.0+0.0i
		;
	double _Complex x =xin;
	if (y < 0)
	{
		y = -y;
		x = 1.0/x;
	}
	for (;;)
	{
		if (y & 1)
			r *= x;
		if (y >>= 1)
			x *= x;
		else
			return r;
	}
}

static double DP1 = 3.14159265160560607910E0;
static double DP2 = 1.98418714791870343106E-9;
static double DP3 = 1.14423774522196636802E-17;
static double _stdcall redupi(double x)
{
	double t;
	long i;

	t = x/M_PI;
	if (t >= 0.0)
		t += 0.5;
	else
		t -= 0.5;
	i = t;
	t = i;
	t = ((x - t *DP1) - t * DP2) - t * DP3;
	return t;
}
#undef catan
double _Complex __declspec(naked) catan(double _Complex z)
{
}
float _Complex __declspec(naked) catanf(float _Complex z)
{
}

long double _Complex EXPORT catanl(const long double _Complex z)
{
	long double a,t,x,x2,y;
	long double _Complex result;

	x = z.re;
	y = z.im;
	if (x == 0.0 && y > 1.0)
		goto overflow;
	x2 = x*x;
	a = 1.0 - x2 - (y * y);
	if (a == 0.0)
		goto overflow;
	t = atan2l(2.0 * x,a)/2.0;
	result.re = redupi(t);
	t = y - 1.0;
	a = x2 + (t * t);
	if (a == 0.0)
		goto overflow;
	t = y + 1.0;
	a = (x2 + (t * t))/a;
	result.im = logl(a)/4.0;
	return result;
overflow:
	a = 0.0;
	result.re = 1.0/a;
	result.im = result.re;
	return result;
}



long double _Complex ctanhl(long double _Complex z)
{
  long double _Complex w;
  long double x, y, d;

  x = creal(z);
  y = cimag(z);
  d = coshl(2.0 * x) + cosl(2.0 * y);
  w = sinhl(2.0 * x) / d  +  (sinl(2.0 * y) / d) * I;
  return (w);
}

long double _Complex cacoshl(long double _Complex z)
{
  long double _Complex w;

  w = I * cacos(z);
  return (w);
}
// CASINH
double _Complex __declspec(naked) casinh(double _Complex q)
{
}
float _Complex __declspec(naked) casinhf(float _Complex q)
{
}
long double _Complex casinhl(long double _Complex z)
{
  long double _Complex w;

  w = -1.0 * I * casinl(z * I);
  return (w);
}

double _Complex __declspec(naked) catanh(double _Complex x)
{
}
long double _Complex EXPORT catanhl(long double _Complex x)
{
        long double _Complex tmp,result;

        tmp.re = 0.0;
        tmp.im = 1.0;
        tmp = x * tmp;
        result = catanl(tmp);
        tmp.re = 0.0;
        tmp.im = -1.0;
        result *= tmp;
        return result;
}

long double _Complex ccoshl(long double _Complex z)
{
  long double _Complex w;
  long double x, y;

  x = creal(z);
  y = cimag(z);
  w = coshl(x) * cosl(y)  +  (sinhl(x) * sinl(y)) * I;
  return (w);
}

double _Complex make_complex(double re,double im)
{
	double _Complex result;

	result.re = re;
	result.im = im;
	return result;
}


#ifdef TEST
int main(void)
{
	double _Complex c1,c2;
	iostream *cout;

#ifdef __LCC__
	cout = iostream_new_from_file(stdout);
#endif
	c1 = make_complex(1.0,1.0);
	// Addition
	c2 = c1+c1;
	if (c2 != 0) {
		printf("ok\n");
	}
	cout << c2 << nl;
	c2 += c1;
	cout << c2 << nl;
	// Substraction
	c2 -= c2;
	if (c2 != 0.0)
		printf("*** c2 should be zero!\n");
	// Multiplication
	c2 = c1;
	c2 = c2 *c2;
	cout << c2 << nl;
	// Division
	if (c1/c1 != 1.0)
		printf("**** c1/c1 should be one!\n");
	c2 = 1.0/c1;
	cout << c2 << nl;
	cout << "Complex square root of (1,0) ";
	cout << csqrt(c1) << nl;
	cout << "Complex cosh of (1,0) ";
	cout << ccosh(c1) << nl;
	c1.re = 0.5;
	c1.im = 0.5;
	cout << "Complex atanh of (0.5,0.5)" ;
	cout << catanh(c1) << nl;
	return 0;
}
#endif
#undef csin
double _Complex csin(double _Complex z)
{
  double _Complex w;
  long double ch, sh;

  cchshl((long double)cimag (z), &ch, &sh );
  w = sin (creal(z)) * ch + (cos (creal(z)) * sh) * I;
  return (w);
}
#undef ccos
double _Complex ccos(double _Complex z)
{
  double _Complex w;
  long double ch, sh;

  cchshl((long double) cimag(z), &ch, &sh );
  w = cos(creal (z)) * ch - (sin (creal (z)) * sh) * I;
  return (w);
}
#undef ctan
double _Complex ctan (double _Complex z)
{
  double _Complex w;
  double d;

  d = cos (2.0 * creal (z)) + cosh (2.0 * cimag (z));

  if (fabs(d) < 0.25)
    d = ctans ((long double _Complex)z);

  if (d == 0.0 )
    {
      w.re = infinity();
      w.im = infinity();
      return (w);
    }

  w = sin (2.0 * creal(z)) / d + (sinh (2.0 * cimag(z)) / d) * I;
  return (w);
}
#undef ccosh
double _Complex ccosh(double _Complex z)
{
  double _Complex w;
  double x, y;

  x = creal(z);
  y = cimag(z);
  w = cosh (x) * cos (y)  +  (sinh (x) * sin (y)) * I;
  return (w);
}
#undef csinh
double _Complex csinh (const double _Complex x)
{
        double _Complex result;

        result.re = sinh(x.re) * cos(x.im);
        result.im = cosh(x.re) * sin(x.im);
        return result;
}
#undef ctanh
double _Complex ctanh (double _Complex z)
{
  double _Complex w;
  double x, y, d;

  x = creal(z);
  y = cimag(z);
  d = cosh (2.0 * x) + cos (2.0 * y);
  w = sinh (2.0 * x) / d  +  (sin (2.0 * y) / d) * I;
  return (w);
}
#undef cacosh
double _Complex cacosh (double _Complex z)
{
  double _Complex w;

  w = I * cacosl(z);
  return (w);
}


