#include <complex.h>

# define M_PI_2         1.57079632679489661923  /* pi/2 */

static void asinr(long double a,long double *re,long double *im)
{				/* z = arcsin(a) */
	long double _Complex z;

	if (fabs (a) <= 1.0) {
		*re = asin(a);
		*im = 0;
	}
	else {
		if (a < 0.0)	{
			*re = -M_PI_2;
			*im = acoshl(-a);
		}
		else {
			*re = M_PI_2;
			*im = acoshl(a);
		}
	}
}

#undef casin

double _Complex EXPORT casin(double _Complex a)
{				/* z = arcsin(a) */
	double R = a.re, I = a.im;
	double _Complex z;
	long double re,im;

	if (I == 0)  {
		asinr((long double)R,&re,&im);
		z.re = re;
		z.im = im;
	}
	else {
		double x = fabs (R), y = fabs (I);
		double r = hypot (x + 1, y), s = hypot (x - 1, y);
		double A = 0.5 * (r + s);
		double B = x / A;
		double y2 = y * y;

		double real, imag;

		const double A_crossover = 1.5, B_crossover = 0.6417;

		if (B <= B_crossover) {
			real = asin (B);
		}
		else {
			if (x <= 1) {
				double D = 0.5 * (A + x) * (y2 / (r + x + 1) + (s + (1 - x)));
				real = atan (x / sqrt (D));
			}
			else {
				double Apx = A + x;
				double D = 0.5 * (Apx / (r + x + 1) + Apx / (s + (x - 1)));
				real = atan (x / (y * sqrt (D)));
			}
		}

		if (A <= A_crossover) {
			double Am1;

			if (x < 1) {
				Am1 = 0.5 * (y2 / (r + (x + 1)) + y2 / (s + (1 - x)));
			}
			else {
				Am1 = 0.5 * (y2 / (r + (x + 1)) + (s + (x - 1)));
			}

			imag = log1pl(Am1 + sqrt (Am1 * (A + 1)));
		}
		else {
			imag = log (A + sqrt (A * A - 1));
		}
		z.re = (R >= 0) ? real : -real;
		z.im = (I >= 0) ? imag : -imag;
	}
	return z;
}

long double _Complex EXPORT casinl(long double _Complex a)
{				/* z = arcsin(a) */
	long double R = a.re, I = a.im;
	long double _Complex z;

	if (I == 0)  {
		asinr(R,&z.re,&z.im);
	}
	else {
		long double x = fabs(R), y = fabs (I);
		long double r = hypotl(x + 1, y), s = hypotl(x - 1, y);
		long double A = 0.5L * (r + s);
		long double B = x / A;
		long double y2 = y * y;
		long double real, imag;

		const double A_crossover = 1.5, B_crossover = 0.6417;

		if (B <= B_crossover) {
			real = asinl(B);
		}
		else {
			if (x <= 1) {
				long double D = 0.5L * (A + x) * (y2 / (r + x + 1) + (s + (1 - x)));
				real = atanl(x / sqrtl(D));
			}
			else {
				long double Apx = A + x;
				long double D = 0.5L * (Apx / (r + x + 1) + Apx / (s + (x - 1)));
				real = atanl(x / (y * sqrtl(D)));
			}
		}

		if (A <= A_crossover) {
			long double Am1;

			if (x < 1) {
				Am1 = 0.5L * (y2 / (r + (x + 1)) + y2 / (s + (1 - x)));
			}
			else {
				Am1 = 0.5L * (y2 / (r + (x + 1)) + (s + (x - 1)));
			}

			imag = log1pl(Am1 + sqrtl(Am1 * (A + 1)));
		}
		else {
			imag = logl(A + sqrtl(A * A - 1));
		}
		z.re = (R >= 0) ? real : -real;
		z.im = (I >= 0) ? imag : -imag;
	}
	return z;
}

float _Complex EXPORT casinf(float _Complex f)
{
	long double _Complex s;
	float _Complex result;

	s.re = f.re;
	s.im = f.im;
	s = casinl(s);
	result.im = s.im;
	result.re = s.re;
	return result;
}

long double _Complex EXPORT cacosl(long double _Complex z)
{

	long double _Complex w = casinl(z);
	w.re = M_PI_2l  -  w.re;
	w.im = -w.im;
	return w;
}
#undef cacos
double _Complex EXPORT cacos(double _Complex z)
{
	double _Complex w = casin(z);
	w.re = M_PI_2 - w.re;
	w.im = -w.im;
	return w;
}

float _Complex EXPORT cacosf(float _Complex z)
{
	double _Complex tmp;
	float _Complex r;
	tmp.re = z.re;
	tmp.im = z.im;
	tmp = casin(tmp);
	r.re = tmp.re;
	r.im = tmp.im;
	return r;
}
