#include <seh.h>
#include <stdlib.h>
#include <windows.h>

#define require(constraint) \
   ((constraint) ? 1 : ConstraintFailed(__func__,#constraint,NULL) )

enum {
	USER_EXCEPTION_OFFSET = 0xe0000000,
	PRECONDITION_VIOLATION =(USER_EXCEPTION_OFFSET + 1),
	POSTCONDITION_VIOLATION =(USER_EXCEPTION_OFFSET + 20),
	INVARIANT_VIOLATION    =(USER_EXCEPTION_OFFSET + 40),
	CHECK_VIOLATION         =(USER_EXCEPTION_OFFSET + 50),
	NO_MEMORY				=(USER_EXCEPTION_OFFSET + 60),
	INVALID_POINTER_OP		=(USER_EXCEPTION_OFFSET + 65),
	USER_EXCEPTION_TEST    =(USER_EXCEPTION_OFFSET + 100)
};

enum {MAX_HANDLERS = 256,
	  MAX_EXCEPTIONS = 5,
	  MAX_RAISE_EXCEPTION_ARGUMENTS};

struct Exception {
	unsigned long code;
	char file_name[MAX_PATH];
	char function_name[MAX_PATH];
	long line;
	void *other_data; // does that make sense?
};

typedef void (*ExceptionHandler)(struct Exception*);

struct ExceptionGlobals {
	unsigned long RaiseExceptionArguments[MAX_RAISE_EXCEPTION_ARGUMENTS];
	struct Exception *ExeptionsField [MAX_EXCEPTIONS];
	ExceptionHandler Handlers[MAX_HANDLERS];
};
extern struct ExceptionGlobals GLOBALS;

void DefaultHandlerWarn (struct Exception*);
void DefaultHandlerAbort(struct Exception*);
void DefaultHandlerExit(struct Exception*, int);
struct Exception * GetAnExceptionStructure (void);
int RegisterHandler (unsigned long place, ExceptionHandler handler);
int DeregisterHandler (unsigned long place);
void InitializeExceptions(void);
ExceptionHandler GetException(unsigned long code);
int HandlerIndexInRange(unsigned long code);
void RunHandler(unsigned long);
void Strabort(char *func,int code,char *expr);

#if 0
#define EXCEPTION_TEMPLATE(test, exception_code, exc_struct) \
	do { \
		if (! (test)) { \
			exc_struct->code = exception_code; \
			strncpy(exc_struct->file_name, __FILE__, MAX_PATH);  \
			strncpy(exc_struct->function_name, __func__, MAX_PATH);  \
			exc_struct->line = __LINE__; \
 			GLOBALS.RaiseExceptionArguments[0] = (unsigned) exc_struct; \
			RaiseException((exc_struct->code),0, \
			1, (DWORD *)exc_struct); \
		} \
	} while (0)
#else
#define EXCEPTION_TEMPLATE(test,exception_code,exc_struct) \
	do { \
		if (!test) { \
			Strabort(__func__,exception_code,#test); \
		} \
	} while (0)
#endif

#ifndef NDEBUG
#define PRECONDITION(test, exc) EXCEPTION_TEMPLATE((test), PRECONDITION_VIOLATION, (exc))
#define POSTCONDITION(test, exc) EXCEPTION_TEMPLATE((test), POSTCONDITION_VIOLATION, (exc))
#define CHECK(test, exc) EXCEPTION_TEMPLATE((test), CHECK_VIOLATION, (exc))
#define SETEXCEPTIONS(exc) struct Exception *exc = GetAnExceptionStructure()
#else
#define PRECONDITION(test, exc)
#define POSTCONDITION(test, exc)
#define CHECK(test,exc)
#define SETEXCEPTIONS(exc)
#endif




