#ifndef __list_h__
#define __list_h__
#include <stddef.h>
#include <stdbool.h>
#include "containererror.h"
/* ---------------------------------------------------------
   LIST Interfaces header file.
   The list type is an opaque data type. Clients aren't
   allowed to use the internal structure of the vectors, only
   the interface to them.
   This file describes the single linked list data structure functions
*/
typedef struct _link {
	struct _link *Next;
	char Data[];
} link;
// The compare function
typedef int (*ComparisonFunction)(void *elem1,void *elem2);
typedef struct _List List;
typedef struct {
        // Returns the number of elements stored
        int (*GetCount)(List *AL);

        // Is this collection read only?
        int (*IsReadOnly)(List &AL);

        // Sets this collection read-only or unsets the read-only flag
        int (*SetReadOnly)(List &AL,int flag);

        // Adds one element at the end. Given element is copied
        int (*Add)(List &AL,void *newval);

        // Clears all data and frees the memory
        int (*Clear)(List &AL);

        //Case sensitive search of a character string in the data
        int (*Contains)(List &AL,void *str);

        // Copies all strings into a NULL terminated vector
        List (*Copy)(List &AL);

        //Returns the index of the given string or -1 if not found
        int (*IndexOf)(List &AL,void *SearchedElement);

        // Inserts a string at the position zero.
        int (*Insert)(List &AL,void *);

        // Inserts a string at the given position
        int (*InsertAt)(List &AL,int idx,void *newval);

        // Returns the string at the given position
        void *(*IndexAt)(List *AL,int idx);

        // Removes the given string if found
        int (*Remove)(List &AL,void *);

        //Removes the string at the indicated position
        int (*RemoveAt)(List &AL,int idx);

        // Frees the memory used by the collection
        int (*Finalize)(List &AL);

        // Pushes a string, using the collection as a stack
        int (*Push)(List &AL,void *str);

        // Pops the last string off the collection
        void * (*Pop)(List &AL);

        // Replaces the character string at the given position with a new one
        void *(*ReplaceAt)(List &AL,int idx,void *newval);

	// Sorts the list
	List (*Sort)(List &l,int (*compar)(const void *,const void *));

	// Reverses the list
	List (*Reverse)(List &l);

	// Gets an element range from the list
	List (*GetRange)(List &l,size_t start,size_t end);

	// Compares two list (Shallow comparison)
	bool (*Equal)(List &l1,List &l2);

	// Add a list at the end of another
	List (*Append)(List &l1,List &l2);

	// Comparison function
	int (*CompareFn)(void * elem1, void *elem2);

	// Set The comparison function
	ComparisonFunction (*SetCompareFunction)(List &l,ComparisonFunction fn);

} ListFunctions;

struct _List {
	ListFunctions *lpVtbl;
	size_t ElementSize;
	size_t count;                    /* in elements units */
	link *Last;                     /* The last item */
       	link *First;                    /* The contents of the list start here */
        unsigned Flags;
};
List newList(int);
List newStringList(void);
void * operator[]=(List &l,size_t position,void *data);
void * operator[](List &l,int position);
#endif
