#include "str.h"
/** Finds the first occurence of @a element in @a string.
@param string to be searched for @a element.
@param element will get searched for
@returns A valid string if found; otherwise an invalid string
*/
#include "exceptions.h"
#ifndef EXPORT
#define EXPORT
#endif
StringpA EXPORT overloaded Strchr(StringA & string, int element)
{
	StringpA result;
	if (!StrvalidA(string))
		return invalid_stringpA;
	char *p = memchr(string.content,element,string.count);
	if (p){ // found
		result.count = string.count - (p - string.content);
		result.content = p;
		result.parent = &string;
		return  result;
	}
	return invalid_stringpA;
}

StringpW EXPORT overloaded Strchr(StringW & string, int element)
{
	StringpW result;
	if (!StrvalidW(string))
		return invalid_stringpW;
	wchar_t *p = wmemchr(string.content,element,string.count);
	if (p) { // found
		result.count = string.count - (p - string.content);
		result.content = p;
		result.parent = &string;
	}
	return invalid_stringpW;
}

StringpA EXPORT overloaded Strchr(StringA & string, int element, int startidx)
{
	StringpA result;
	if (!StrvalidA(string) || startidx < 0 || startidx >= string.count)
		return invalid_stringpA;
	char *p = memchr(string.content+startidx,element,string.count-startidx);
	if (p) { // found
		result.count = string.count - (p - string.content);
		result.content = p;
		result.parent = &string;
		return result;
	}
	return invalid_stringpA;
}

StringpW EXPORT overloaded Strchr(StringW & string, int element, int startidx)
{
	StringpW result;
	if (!StrvalidW(string) || startidx < 0 || startidx >= string.count)
		return invalid_stringpW;
	wchar_t *p = wmemchr(string.content+startidx,element,string.count-startidx);
	if (p) { // found
		result.count = string.count - (p - string.content);
		result.content = p;
		result.parent = &string;
		return result;
	}
	return invalid_stringpW;
}

StringpA EXPORT overloaded Strchr(StringpA & stringp, int element)
{
	StringpA result;
	char *p;

	if (stringp.count == 0 || stringp.parent == NULL ||
		stringp.content == NULL)
		return invalid_stringpA;
	p = memchr(stringp.content,element,stringp.count);
	if (p) {
		result.count = stringp.count - (p - stringp.content);
		result.content = p;
		result.parent = stringp.parent;
	}
	else {
		result.parent=NULL;
		result.content=NULL;
		result.count=0;
	}
	return result;
}

StringpW EXPORT overloaded Strchr(StringpW & stringp, int element)
{
	StringpW result;
	wchar_t *p;

	if (stringp.count == 0 || stringp.parent == NULL ||
		stringp.content == NULL)
		return invalid_stringpW;
	p = wmemchr(stringp.content,element,stringp.count);
	if (p) {
		result.count = stringp.count - (p - stringp.content);
		result.content = p;
		result.parent = stringp.parent;
	}
	else {
		result.parent=NULL;
		result.content=NULL;
		result.count=0;
	}
	return result;

}

/** Finds the last occurence of @a element in @a string.
@param string to be searched from the end for @a element.
@param element find this element.
@returns A valid String if found otherwise an invalid String
@notes If the result is a valid String, it will always be a fresh copy
but not the original string
*/
StringpA EXPORT overloaded Strrchr(StringA & string, int element)
{
	int i;
	char *p;
	StringpA result;

	if (string.content == NULL || string.capacity <= string.count)
		return invalid_stringpA;
	p=string.content + string.count - 1;
	for (i = string.count; i >0; --i){
		if (*p-- == element){
			result.count = string.count-i+1;
			result.content = string.content+i-1;
			result.parent = &string;
			return result;
		}
	}
	return invalid_stringpA;
}


StringpW EXPORT overloaded Strrchr(StringW & string, int element)
{
	int i;
	wchar_t *p;
	StringpW result;

	if (!StrvalidW(string))
		return invalid_stringpW;
	p=string.content + string.count - 1;
	for (i = string.count; i >0; --i){
		if (*p-- == element){
                	result.count = string.count-i+1;
                	result.content = string.content+i-1;
                	result.parent = &string;
			return result;
        	}
	}
	return invalid_stringpW;
}
