#include <assert.h>
#include "str.h"
#include "exceptions.h"
#include "str_private.h"

/** Make a new copy from @a to_be_cloned
@param to_be_cloned this string gets copied.
*/
StringA EXPORT overloaded Strdup(StringA & to_be_cloned)
{
	StringA result;
	int code = 0;
	require(StrvalidA(to_be_cloned));

	result = to_be_cloned;
	result.content = GC_malloc(result.capacity);
	if (result.content == NULL) {
		require(0);
		return invalid_stringA;
	}
	memcpy(result.content,to_be_cloned.content,to_be_cloned.count+1);
	result.content[result.count] = 0;
	return result;
}

StringA EXPORT overloaded Strdup(char *txt)
{
	return new_stringA(txt);
}

StringW EXPORT overloaded Strdup(wchar_t * to_be_cloned){
	return new_stringW(to_be_cloned);

}

StringW EXPORT overloaded Strdup(StringW & to_be_cloned)
{
	StringW result;
	require(StrvalidW(to_be_cloned));

	result = to_be_cloned;
	result.content = GC_malloc(sizeof(wchar_t)*result.capacity);
	if (result.content == NULL) {
		require(0);
		return invalid_stringW;
	}
	memcpy(result.content,to_be_cloned.content,sizeof(wchar_t)*(to_be_cloned.count+1));
	result.content[result.count] = 0;
	return result;
}

StringA EXPORT overloaded Strdup(StringpA to_be_cloned)
{
	StringA result;
	int code = 0;
	require(Strvalid(*to_be_cloned.parent));

	result.count = to_be_cloned.count;
	result.content = GC_malloc(to_be_cloned.parent->capacity);
	if (result.content == NULL) {
		require(result.content != NULL);
		return invalid_stringA;
	}
	memcpy(result.content,to_be_cloned.content,result.count+1);
	return result;
}

StringW EXPORT overloaded Strdup(StringpW to_be_cloned)
{
	StringW result;
	int code = 0;
	require(StrvalidW(*to_be_cloned.parent));

	result.count = to_be_cloned.count;
	result.content = GC_malloc(to_be_cloned.parent->capacity*sizeof(wchar_t));
	if (result.content == NULL) {
		require(result.content != NULL);
		return invalid_stringW;
	}
	memcpy(result.content,to_be_cloned.content,sizeof(wchar_t)*(result.count+1));
	return result;
}

StringA EXPORT overloaded Strdup(StringA &str,int start)
{
	if (!StrvalidA(str) || start < 0 || start >= str.count-1)
		return invalid_stringA;
	return Strdup(str.content+start);
}

StringW EXPORT overloaded Strdup(StringW &str, int start)
{
	if (!StrvalidW(str) || start < 0 || start >= str.count-1)
		return invalid_stringW;
	return Strdup(str.content+start);
}

StringW EXPORT overloaded Strdup(StringW &str, int start,int end)
{
	StringW result;

	if (!StrvalidW(str) || start < 0 || start >= str.count-1 || end < start)
		return invalid_stringW;
	if (end >= str.count-1)
		end = str.count-1;

	result.count = 1+end-start;
	result.capacity = result.count+1;
	result.content = GC_malloc(sizeof(wchar_t)*result.capacity);
	if (result.content == NULL) {
		require(result.content != NULL);
		return invalid_stringW;
	}
	memcpy(result.content,str.content+start,sizeof(wchar_t)*result.count);
	return result;
}

StringA EXPORT overloaded Strdup(StringA &str, int start,int end)
{
	StringA result;

	if (!StrvalidA(str) || start < 0 || start >= str.count-1 || end < start)
		return invalid_stringA;
	if (end >= str.count-1)
		end = str.count-1;

	result.count = 1+end-start;
	result.capacity = 1+result.count;
	result.content = GC_malloc(result.capacity);
	if (result.content == NULL) {
		require(result.content != NULL);
		return invalid_stringA;
	}
	memcpy(result.content,str.content+start,result.count);
	return result;
}

