#include "str.h"
#include "exceptions.h"
#include "str_private.h"

/** Retrieve a string from the contents of 'file_name'
*/
StringA EXPORT overloaded Strfromfile (char * file_name,int binarymode)
{
	FILE *fp;
	StringA result;

	if (binarymode)
		 fp = fopen(file_name, "rb");
	 else
		  fp = fopen(file_name, "r");
	if (NULL == fp){
		return invalid_stringA;
	}
	size_t needed = 0;
	int i_rval = 0;
	i_rval = fseek(fp, 0, SEEK_END);
	if (0 != i_rval) {
		fclose(fp);
		return invalid_stringA;
	}
	needed = ftell(fp);
	i_rval = fseek(fp, 0, SEEK_SET);
	if (0 != i_rval){
		fclose(fp);
		return invalid_stringA;
	}
	result.content = allocate_proper_space(needed+1,sizeof(*(result.content)));
	if (result.content == NULL)
		return invalid_stringA;
	result.capacity = needed+1;
	size_t u_rval = fread(result.content, 1, needed, fp);
	fclose(fp);
	if (u_rval != needed && binarymode){
		return invalid_stringA;
	}
	result.count = u_rval;
	result.content[result.count] = 0;
	return result;
}
StringA EXPORT overloaded Strfromfile(StringA file_name,int binarymode)
{
	return Strfromfile((char *)file_name.content,binarymode);
}

StringA EXPORT overloaded Strfromfile(char *file_name)
{
	return Strfromfile(file_name,0);
}

StringA EXPORT overloaded Strfromfile(String file_name)
{
	return Strfromfile(file_name.content,0);
}

bool StrtofileA(StringA &s,char *filename,int binarymode)
{
	FILE *f;
	int r;
	if (filename == NULL) {
		RunHandler(INVALID_ARG);
		return false;
	}
	if (!StrvalidA(s)) {
		RunHandler(INVALID_ARG);
		return false;
	}
	f = fopen(filename,binarymode ? "rb" : "r");
	if (f == NULL) {
		RunHandler(INVALID_FILE_OPERATION);
		return false;
	}
	r = fwrite(f,1,s.count,f);
	if (r != s.count) {
		RunHandler(INVALID_FILE_OPERATION);
		fclose(f);
		return false;
	}
	fclose(f);
	return true;
}
bool StrtofileW(StringW &s,char *filename,int binarymode)
{
	FILE *f;
	int r;
	if (filename == NULL) {
		RunHandler(INVALID_ARG);
		return false;
	}
	if (!StrvalidW(s)) {
		RunHandler(INVALID_ARG);
		return false;
	}
	f = fopen(filename,binarymode ? "rb" : "r");
	if (f == NULL) {
		RunHandler(INVALID_FILE_OPERATION);
		return false;
	}
	r = fwrite(f,sizeof(wchar_t),s.count,f);
	if (r != s.count) {
		RunHandler(INVALID_FILE_OPERATION);
		fclose(f);
		return false;
	}
	fclose(f);
	return true;
}

/** @defgroup IO Input/Output
@{ */

/** Copies maximal @a n elements from the FILE @a from to @a source.
@param source read into this string.
@param n maximal this many elements.
@param from this source.
@notes @a source will be always check if it's large enough to take
@a n elements. If neccessary the buffer gets resized.
The reading stop if n or a newline is reached. If there are
more element read than n character @a source will get
cut. But it will always be a proper terminated string,
which can be used by other C functions.
The function is just for reading char not
wchar_t types.
*/

/** See examples in @example examples/Strfgets_examples.c
*/

int overloaded Strfgets(StringA & source, int n, FILE *from)
{
	int i,val;
	if (StrvalidA(source) == 0 || n > source.capacity) {
		source.content = allocate_proper_space(n+1, sizeof(*(source.content)));
		if (source.content == NULL)
			return 0;
		source.capacity = n+1;
	}

	for (i = 0; i < n; ) {
		val = fgetc(from);
		if (val == EOF || val == '\n')
			break;
		if (val != '\r') {
			source.content[i++] = val;
		}
	}
	source.content[i] = 0;
	source.count = i;
	return (i);
}

StringA overloaded Strfgets(FILE *from)
{
	StringA result = {0};
	char *p=NULL;
	int len = fgetstralloc(&p,from);

	if (len <=0) {
		if (p)
			free(p);
		return invalid_stringA;
	}
	if (p[len-1] == '\n') {
		len--;
	}
	result.content = allocate_proper_space(len+1,sizeof(char));
	strncpy(result.content,p,len);
	free(p);
	result.count = len;
	result.capacity = len+1;
	return result;
}
