/** Strlib.c
@author Friedrich Dominicus/Jacob Navia date 2002
@verston 1.1 summer 2004
*/
#include <locale.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include <gc.h>
#include <stdbool.h>
#include <wchar.h>
#include <windows.h>
#include "str.h"
#include "exceptions.h"
#include "str_private.h"
#include <seh.h>


#ifdef __DLL__
#define EXPORT __declspec(dllexport)
#else
#define EXPORT
#endif


#ifdef UNICODE
#define invalid_string invalid_stringW
#else
#define invalid_string invalid_stringA
#endif
/** Prototypes */


/* private */
static void set_count (String &, size_t);
static void set_capacity (String &, size_t);
void overloaded set_content (String& str, String);
static void overloaded set_content (String& str, void*);
void * allocate_proper_space (size_t, unsigned int);
size_t calculate_new_capacity (size_t, size_t);



int isNullStringA(StringA &s)
{
	return s.count == 0 && s.content == NULL && s.capacity == 0;
}
int isNullStringW(StringW &s)
{
	return s.count == 0 && s.content == NULL && s.capacity == 0;
}

int isNullStringpA(StringpA &s)
{
	return s.count == 0 && s.content == NULL && s.parent == NULL;
}

int isNullStringpW(StringpW &s)
{
	return s.count == 0 && s.content == NULL && s.parent == NULL;
}


int overloaded Strsizeof(StringA s)
{
	int result = s.capacity;
	result += sizeof(StringA);
	return result;
}
int overloaded Strsizeof(StringW s)
{
	int result = s.capacity*sizeof(wchar_t);
	result += sizeof(String);
	return result;
}

char * EXPORT overloaded Strcontent(StringA &a)
{
	require(Strvalid(a));

	return a.content;
}

wchar_t * EXPORT overloaded Strcontent(StringW &a)
{
	int len;
	wchar_t *result;
	require(StrvalidW(a));

	return a.content;
}

#ifdef strlen
size_t mstrlen (char* src){
	return strlen(src);
}
#else
size_t mstrlen(char* src){
	return strlen(src);
}
#endif



static enum {BUF_SIZE=128};
/** Create a new String form @init.
@param init initialize String with it's content

*/
String EXPORT overloaded ToString(int init,char *fmt)
{
	char buf[BUF_SIZE];
	snprintf(buf,BUF_SIZE, fmt, init);
	return new_stringA(buf);
}

String EXPORT overloaded ToString(int init)
{
	char buf[BUF_SIZE];
	snprintf(buf,BUF_SIZE, "%d", init);
	return new_stringA(buf);
}
/** Create a new String from an unsigned int.
@param init initialize String with it's content.
*/
String EXPORT overloaded ToString(unsigned int init,char *fmt)
{
	char buf[BUF_SIZE];
	snprintf(buf, BUF_SIZE, fmt, init);
	return new_stringA(buf);
}

String EXPORT overloaded ToString(unsigned int init)
{
	char buf[BUF_SIZE];
	snprintf(buf, BUF_SIZE, "%ud", init);
	return new_stringA(buf);
}

String EXPORT overloaded ToString(char* from){
	return new_stringA(from);
}


/** Create Stringf from a float.
*/
String EXPORT overloaded ToString(float init,char *fmt)
{
	char buf[BUF_SIZE];
	snprintf(buf, BUF_SIZE, fmt, init);
	return new_stringA(buf);
}

String EXPORT overloaded ToString(float init)
{
	char buf[BUF_SIZE];
	snprintf(buf, BUF_SIZE, "%g", init);
	return new_stringA(buf);
}

/** Create String from a double.
*/
String EXPORT overloaded ToString(double init,char *fmt = "%g")
{
	char buf[BUF_SIZE];
	snprintf(buf,BUF_SIZE, fmt, init);
	return new_stringA(buf);
}
String EXPORT overloaded ToString(double init)
{
	char buf[BUF_SIZE];
	snprintf(buf,BUF_SIZE, "%g", init);
	return new_stringA(buf);
}

/** Create String from a long double.
*/
String EXPORT overloaded ToString(long double init,char *fmt)
{
	char buf[BUF_SIZE];
	sprintf(buf, fmt, init);
	return new_stringA(buf);
}


/** Create a string from a long long.
*/
String EXPORT overloaded ToString(long long init,char *fmt="%lld")
{
	char buf[BUF_SIZE];
	snprintf(buf, BUF_SIZE,fmt, init);
	return new_stringA(buf);
}


/** Convert string of type SMALL to a string of type WIDE.
Does nothing if string is still of type WIDE.
@param string gets converted to a wide character string.c
*/
StringW EXPORT Strtowide(StringA & string)
{
	size_t content_length = 0;
	int code = 0;
	StringW result;
	require(Strvalid(string));

	char *content = string.content;
	content_length = string.count;
	wchar_t * outbuf = allocate_proper_space(content_length + 1, sizeof(*outbuf));
	if (outbuf == NULL)
		return invalid_stringW;
	int ret_val = MultiByteToWideChar(GetACP(),
		MB_PRECOMPOSED,
		content,
		content_length,
		(LPWSTR) outbuf,
		content_length);

	require(ret_val != 0);
	result.content = outbuf;
	result.count = ret_val;
	require(string.count >= content_length);
	return result;
}

/** Convert string of type WIDE to a string of type SMALL.
Does nothing if string is still of type SMALL.
@param string this string gets converted. If conversion failed it will
be unchanged
*/
StringA EXPORT Strtosmall(StringW & string)
{
	require(StrvalidW(string));
	size_t content_length = 0;
	int code = 0;
	StringA result;

	int ret_val = 0;
	wchar_t * content = string.content;
	content_length = string.count;
	char *outbuf = allocate_proper_space(content_length+1, sizeof(*outbuf));

	if (outbuf == NULL)
		return invalid_stringA;
	ret_val = WideCharToMultiByte(GetACP(),
		0, // unused flag
		content, // field to be converted
		content_length, // length of field
		outbuf, // converted field
		content_length, // converted field length
		NULL, NULL);
	require(0 != ret_val);
	result.content = outbuf;
	result.count = ret_val;
	result.capacity = content_length + 1;
	require(result.count >= content_length);
	return result;
}

/** @} End Conversoins */



/** @addtogroup Accessors Accessor Functions
@{
*/

/** Get the length of @a string.
In comparison to C this is a O(1) operation
@param string take the length of this String.
@returns The @a count element of a @a String structure,
this is the lenght of the string.
*/
//inline
size_t overloaded EXPORT Strlen (const StringA & string)
{
	require(StrvalidA(string));
	return (string.count);
}

size_t overloaded EXPORT Strlen (const StringW &string)
{
	require(StrvalidW(string));
	return (string.count);
}

/** Get the capacity of @a string.
The capacity is the theoreticall maximum which can be contained
in a string before resizing must take place.
@param string get the capacity of this string.
@returns The number of potential element of @a string
*/
size_t EXPORT overloaded Strcapacity(const StringA & string)
{
	require(StrvalidA(string));
	return (string.capacity);
}
size_t EXPORT overloaded Strcapacity(const StringW & string)
{
	return (string.capacity/2);
}

static void proper_order (int *low, int *high)
{
	int temp = 0;
	if (*low > *high){
		temp = *low;
		*low = *high;
		*high = temp;
	}
}

StringA EXPORT overloaded Strslice(StringA & string, int from, int to)
{
	StringA result = {0};
	int code = 0;

	require(StrvalidA(string));
	int ifrom = from;
	int ito = to;

	proper_order(&ifrom, &ito);
	if (ito > string.count) {
		ito = string.count;
	}
	if (ifrom < 0) {
		ifrom = 0;
	}
	int count = ito - ifrom;

	result = str_newA(count+1, invalid_stringA);
	result.count = count;

	for (int i = 0; i < count; ++i){
		result.content[i] = string.content[i + ifrom];
	}
	result.content[count] = '\0';
	return result;
}

StringW EXPORT overloaded Strslice(StringW & string, int from, int to)
{
	StringW result = {0};
	int code = 0;

	require(StrvalidW(string));
	int ifrom = from;
	int ito = to;

	proper_order(&ifrom, &ito);
	if (ito > string.count) {
		ito = string.count;
	}
	if (ifrom < 0) {
		ifrom = 0;
	}
	int count = ito - ifrom;

	result = str_newW(count+1, invalid_stringW);
	result.count = count;

	for (int i = 0; i < count; ++i){
		result.content[i] = string.content[i + ifrom];
	}
	result.content[count] = '\0';
	return result;
}


#ifdef toupper // is touper a macro?
int to_upper (int ch){
	return (toupper (ch));
}
#endif

StringA EXPORT Strupr(StringA& from)
{
	String result = {0};
	int code = 0;
	require(Strvalid(from));
	for (int i = 0; i< from.count;i++) {
		from.content[i] = toupper(from.content[i]);
	}
	return result;
}

StringA EXPORT Strlwr(StringA& from)
{
	String result = {0};
	int code = 0;
	require(Strvalid(from));
	for (int i = 0; i< from.count;i++) {
		from.content[i] = tolower(from.content[i]);
	}
	return result;
}



void Strprint(String & string){
	/* just for testing  */


	printf("content = %s\ncapacity = %ul, length = %ul\n",
		Strcontent(string),
		Strcapacity(string), Strlen(string));
}

/** @} Input Output */

/** @defgroup Finders Finder Functions
@{
*/

/** Check if one of the elements in @a elements can be found in the String.
@param string gets searched for any of the elements of @a elements
@param elements to be found
@param start_from (search) from this position.
returns If none of the elements can be found. -1, otherwise
position of first occurence of one element from elements.

*/
int EXPORT Strhasanyof (String &string, String & elements, int start_from)
{
	int result = -1;
	int code = 0;

	require(Strvalid(string) && Strvalid(elements));
	require(start_from >= 0 &&
			start_from <= Strlen(string));


	for (int i = start_from; i < Strlen(string) && (result == -1); ++i){
		for (int j = 0; j < Strlen(elements) && (result == -1); ++j){
			if (string[i] == elements[j]){
				result = i;
			}
		}
	}
	CHECK((result == -1) ||
			(result >= start_from), exc);
	return result;
}

/** Standard function? */

String EXPORT Strpbrk(String &s1, String &s2){
	String result = invalid_string;
	int code = 0;
	require(Strvalid(s1) && Strvalid(s2));
	int pos = Strhasanyof(s1,s2,0);
	if (pos >= 0){
		result = Strslice(s1,pos,Strlen(s1));
	}
	return result;
}

/** @} */

/** Split string at position @a at
@param string to be splitted.
@param at positon at which the split takes place.
@return Strsimpleplit a structure containing the first
part of the split and the second part
@todo Is it a good idea to put that into an own structure? */
StrsimplesplitA EXPORT overloaded Strsplit(StringA &string, int at)
{
	StrsimplesplitA result = {0};
	int code = 0;
	require(StrvalidA(string));
	require(at >= 0 && at <= string.count);

	result.pre  = Strslice(string, 0, at);
	result.post = Strslice(string, at + 1, string.count);
	return result;
}

StrsimplesplitW EXPORT overloaded Strsplit(StringW &string, int at)
{
	StrsimplesplitW result = {0};
	int code = 0;
	require(StrvalidW(string));
	require(at >= 0 && at <= string.count);

	result.pre  = Strslice(string, 0, at);
	result.post = Strslice(string, at + 1, string.count);
	return result;
}

//-------------------------------------------------strchcat
void EXPORT overloaded Strchcat(StringA &string, int val)
{
	int old_end_str = 0;
	int code = 0;
	require(StrvalidA(string));

	size_t req_size = string.count + 1;
	size_t old_cap = string.capacity;
	size_t new_cap = calculate_new_capacity(old_cap, req_size);

	if (old_cap < new_cap){
		string = str_newA(new_cap, string);
	}
	char *str = string.content;
	old_end_str = req_size - 1;
	str[old_end_str] = val;
	string.count = req_size;
	require(string.content[old_end_str] == val);
}
void EXPORT overloaded Strchcat(StringW &string, int val)
{
	int old_end_str = 0;
	int code = 0;
	require(StrvalidW(string));

	size_t req_size = string.count + 1;
	size_t old_cap = string.capacity;
	size_t new_cap = calculate_new_capacity(old_cap, req_size);

	if (old_cap < new_cap){
		string = str_newW(new_cap, string);
	}
	wchar_t *str = string.content;
	old_end_str = req_size - 1;
	str[old_end_str] = val;
	string.count = req_size;
	require(string.content[old_end_str] == val);
}

/** Fill @a string with @a val, in @a size elements.
@param string will get filled with @val.
@param val fill value
@param size fill this many elements
*/
void EXPORT overloaded Strset(StringA & string, int val, size_t size)
{
	require(StrvalidA(string));
	require(string.capacity > size);
	memset(string.content, (char)val, size);
	string.count = size;
}

void EXPORT overloaded StrsetW(StringW & string, int val, size_t size)
{
	require(StrvalidW(string));
	require(string.capacity > size);
	int i;
	for (i = 0; i < string.count;i++) {
		string.content[i] = val;
	}
	string.count = i;
}

StringA StrerrorA(void)
{
	return new_stringA(strerror(errno));
}

StringW StrerrorW(void)
{
	char *a = strerror(errno);
	int len = strlen(a);
	StringW result = new_stringW(len+1);
	mbstowcs(result.content,a,strlen(a));
	result[len] = 0;
	return result;
}


StringpA overloaded Strnext(StringpA p)
{
	StringA *pParent = p.parent;
	if (isNullStringpA(p))
		return p;
	int idx = p.content - pParent->content;
	if (idx < pParent->count) {
		p.count--;
		p.content++;
		return p;
	}
	return invalid_stringpA;
}

StringpW overloaded  Strnext(StringpW p)
{
	if (isNullStringpW(p))
		return p;
	StringW *pParent = p.parent;
	int idx = p.content - pParent->content;
	if (idx < pParent->count) {
		p.count--;
		p.content++;
		return p;
	}
	return invalid_stringpW;
}


StringpA overloaded Strbegin(StringA &p)
{
	StringpA result;
	if (!StrvalidA(p))
		return invalid_stringpA;
	result.count = p.count;
	result.content = p.content;
	result.parent = &p;
	return result;
}

StringpW overloaded Strbegin(StringW &p)
{
	StringpW result;
	if (!StrvalidW(p))
		return invalid_stringpW;
	result.count = p.count;
	result.content = p.content;
	result.parent = &p;
	return result;
}

