#include "str.h"
#include "exceptions.h"
#include "str_private.h"


StringA str_newA(size_t requested_size,StringA & string)
{
	char *content = NULL;
	StringA result = {0};
	int strvalid=0;

	/* ok? The idea is that the reallocaton of space in string is somwhat
	concentrated, assume for example string catenation, it's very likely
	that more than two String get concatenated, if you do one allocaton after
	the other you nearly always will have to reallocate for the new larger
	string.
	Inspired by a weakness I found in the SmallEiffel strategy. */
	size_t req_size = requested_size + 1;
	/* it should hold anytime str[count] == '\0' */
	size_t new_capacity = calculate_new_capacity(string.capacity,req_size);

	strvalid = Strvalid(string);
	if ((new_capacity <= string.capacity) && strvalid) {
		/* do nothing */
		return string;
	}

	content = allocate_proper_space(new_capacity, sizeof(*content));
	if (content == NULL)
		return invalid_stringA;
	result.content = content;
	result.capacity = new_capacity;
	if (strvalid) {
		memcpy(result.content,string.content,string.count);
		result.count = string.count;
	}
	return result;
}

StringW str_newW(size_t requested_size,StringW & string)
{
	wchar_t *content = NULL;
	StringW result = {0};
	int strvalid;

	size_t req_size = requested_size + 1;
	/* it should hold anytime str[count] == '\0' */
	size_t new_capacity = calculate_new_capacity(string.capacity,req_size);

	strvalid = StrvalidW(string);
	if ((new_capacity == string.capacity) && strvalid) {
		/* do nothing */
		return string;
	}

	content = allocate_proper_space(new_capacity, sizeof(*content));
	if (content == NULL)
		return invalid_stringW;
	result.content = content;
	result.capacity = new_capacity;
	if (strvalid) {
		memcpy(result.content,string.content,string.count*sizeof(*content));
		result.count = string.count;
	}
	return result;

}


/** @defgroup Creators Creator Functions
@{ */
/** Generate a new String
@param initial_size set the lenght of the new string to at least this
number of elements
@param string_type either @a SMALL (ASCII 8-bit) or @a WIDE Sring
(UNICODE)
*/

/** Examples see @example examples/strnew_examples.c
*/
StringA EXPORT overloaded new_stringA(size_t initial_size)
{
	return str_newA(initial_size, invalid_stringA);
}

StringW EXPORT overloaded new_stringW(size_t initial_size)
{
	return str_newW(initial_size, invalid_stringW);
}


StringpA EXPORT overloaded new_stringpA(StringA &string,char *start)
{
	int len;
	Stringp result;

	len = ((char *)start - (char *)string.content);
	result.count = string.count - len;
	result.content = start;
	result.parent = &string;
	return result;
}

StringpW EXPORT overloaded new_stringpW(StringW &string,wchar_t *start)
{
	int len;
	StringpW result;

	len = ((char *)start - (char *)string.content);
	len >>= 1;
	result.count = string.count - len;
	result.content = start;
	result.parent = &string;
	return result;
}


StringpA EXPORT overloaded new_stringpA(StringA &string,int start)
{
	char *bytecontents;
	int len;
	Stringp result;

	bytecontents = string.content;
	bytecontents += start;
	result.content = bytecontents;
	result.count = string.count - start;
	result.parent = &string;
	return result;
}

StringpW EXPORT overloaded new_stringpW(StringW &string,int start)
{
	wchar_t *wcharcontents;
	int len;
	StringpW result;

	wcharcontents = string.content;
	wcharcontents += start;
	result.content = wcharcontents;
	result.count = string.count - start;
	result.parent = &string;
	return result;
}

/** Generate a new string from an char array @a init
@param init 'proper' C-String (character array ending with an '0'.
@returns A new String.
@notes Strlen(new_String) >= strlen(init) if init
is valid */
StringA EXPORT overloaded new_stringA(char* init)
{
	StringA result;
	require(NULL != init);
	int code = 0;

	size_t str_len = strlen(init);
	result = new_stringA(str_len+1);
	strcpy(result.content, init);
	result.count = str_len;
	require(0 == strcmp(result.content, init));
	return result;
}
/* Create a new string from wchar_t Pointer (UNICODE).
@param init. Array to create a new string from
@returns A new string of at least @a wcslen(init) size
*/
StringW EXPORT overloaded new_stringW(wchar_t *init)
{
	StringW result;
	int code = 0;
	require(NULL != init);
	size_t str_len = wcslen(init);
	result = new_stringW(str_len);
	wcsncpy(result.content, init, str_len);
	result.count =  str_len;
	require(0 == wcscmp(result.content, init));
	return result;
}


