#include "str.h"
#include "exceptions.h"
#include "str_private.h"

/** Cast operators.
Extension of lcc-win32 if a Function is not overloaded,
and is called with a different type as declared, than lcc searches
for a cast from that type to the needed type if it can find one
this cast is applied otherwise the compiler flags it
as an type mismatch (AFAIU) frido
*/
StringA EXPORT operator()(char* cstr)
{
	StringA result;
	require(cstr != NULL);
	result.count = strlen(cstr);
	result.capacity = calculate_new_capacity(0,1+result.count);
	result.content = allocate_proper_space(result.capacity,sizeof(*(result.content)));
	if (result.content == NULL) {
		return invalid_stringA;
	}
	strcpy(result.content,cstr);
	return result;
}

StringW EXPORT operator()(wchar_t *cstr)
{
	StringW result;
	require(cstr != NULL);
	result.count = wcslen(cstr);
	result.capacity = calculate_new_capacity(0,1+result.count);
	result.content = allocate_proper_space(result.capacity,*(result.content));
	if (result.content == NULL) {
		return invalid_stringW;
	}
	wcscpy(result.content,cstr);
	return result;
}

char *EXPORT operator()(StringA &str)
{
	if (isNullStringA(str))
		return NULL;
	str.content[str.count] = 0;
	return str.content;
}

wchar_t *EXPORT operator()(StringW &str)
{
	return str.content;
}

char *EXPORT operator()(StringpA &strp)
{
	char *result;

	require(StrvalidpA(strp));
	if (strp.content[strp.count] == 0)
		return strp.content;
	result = allocate_proper_space(strp.count+1,sizeof(*result));
	if (result == NULL)
		return NULL;
	memcpy(result,strp.content,strp.count);
	return result;
}
wchar_t *EXPORT operator()(StringpW &strp)
{
	wchar_t *result;

	require(StrvalidpW(strp));
	result = strp.content;
	if (result[strp.count] == 0)
		return result;
	result = allocate_proper_space(strp.count+1,sizeof(*result));
	if (result == NULL)
		return NULL;
	memcpy(result,strp.content,strp.count * sizeof(*result));
	return result;
}
#if 0
wchar_t *EXPORT operator()(StringW &str)
{
	return str.content;
}
#endif


