#include "str.h"
#include "exceptions.h"
#include "str_private.h"


/* convenience Functions making String handling a bit easier
to understand */
int EXPORT operator == (const StringA & string1, const StringA & string2)
{
	if (isNullStringA(string2)  && isNullStringA(string1))
		return 1;
	if (isNullStringA(string2) || isNullStringA(string1))
		return 0;
	require(StrvalidA(string1) && StrvalidA(string2));
	if (string1.count != string2.count)
		return 0;
	return !memcmp(string1.content,string2.content,string1.count);
}

int EXPORT operator == (const StringW & string1, const StringW & string2)
{
	if (&string2 == &invalid_stringW && &string1 == &invalid_stringW)
		return 1;
	if (&string2 == &invalid_stringW || &string1 == &invalid_stringW)
		return 0;
	require(StrvalidW(string1) && StrvalidW(string2));
	if (string1.count != string2.count)
		return 0;
	return !memcmp(string1.content,string2.content,string1.count*sizeof(wchar_t));
}

int EXPORT operator != (const StringA & string1, const StringA & string2)
{
	if (isNullStringA(string1) && isNullStringA(string2))
		return 0;
	if (isNullStringA(string1) || isNullStringA(string2))
		return 1;
	require(StrvalidA(string1) && StrvalidA(string2));
	if (string1.count != string2.count)
		return 1;
	return 0 != memcmp(string1.content,string2.content,string1.count);
}
int EXPORT operator != (const StringW & string1, const StringW & string2)
{
	if (isNullStringW(string1) && isNullStringW(string2))
		return 0;
	if (isNullStringW(string1) || isNullStringW(string2))
		return 1;
	require(StrvalidW(string1) && StrvalidW(string2));
	if (string1.count != string2.count)
		return 1;
	return 0 != memcmp(string1.content,string2.content,string1.count*sizeof(wchar_t));
}

int EXPORT operator == (const StringpA & string1, int i)
{
	if (i == 0 && isNullStringpA(string1))
		return 1;
	return 0;
}

int EXPORT operator == (const StringpW & string1, int i)
{
	if (i == 0 && isNullStringpW(string1))
		return 1;
	return 0;
}

int EXPORT operator == (int i,const StringpA & string1)
{
	if (i == 0 && isNullStringpA(string1))
		return 1;
	return 0;
}

int EXPORT operator == (int i,const StringpW & string1)
{
	if (i == 0 && isNullStringpW(string1))
		return 1;
	return 0;
}

int EXPORT operator != (const StringpA & string1, int i)
{
	if (isNullStringpA(string1) && i == 0)
		return 0;
	return 1;
}

int EXPORT operator -(StringpA &s1, StringpA &s2)
{
	require(StrvalidpA(s1) && StrvalidpA(s2) && s1.parent == s2.parent);
	return s1.content - s2.content;
}

StringpA EXPORT operator+=(StringpA &s1,int n)
{
	require(StrvalidpA(s1) && n < s1.count);
	s1.count -= n;
	s1.content += n;
	return s1;
}

StringpW EXPORT operator+=(StringpW &s1,int n)
{
	require(StrvalidpW(s1) && n < s1.count);
	s1.count -= n;
	s1.content += n;
	return s1;
}

StringpA EXPORT operator+(StringA &s1,int n)
{
	require(StrvalidA(s1) && n < s1.count);
	StringpA result = {0};

	result.count = s1.count - n;
	result.content = s1.content + n;
	result.parent = &s1;
	return result;
}

StringpW EXPORT operator+(StringW &s1,int n)
{
	require(StrvalidW(s1) && n < s1.count);
	StringpW result = {0};

	result.count = s1.count - n;
	result.content = s1.content + n;
	result.parent = &s1;
	return result;
}

int __declspec(naked) lessStringA(const StringA  *string1,const StringA *string2)
{
}
int EXPORT operator < (const StringA & string1, const StringA &string2)
{
	bool s1null = isNullStringA(string1);
	bool s2null = isNullStringA(string2);
	if (s1null && s2null)
		return 0;
	if (s1null && !s2null)
		return 1;
	if (!s1null && s2null)
		return 0;
	require(StrvalidA(string1) && StrvalidA(string2));
	if (string1.count == 0 && string2.count != 0)
		return 1;
	if (string1.count && string2.count == 0)
		return 0;
	return strcmp(string1.content, string2.content) < 0;
}

int __declspec(naked) lessStringW(const StringW  *string1,const StringW *string2)
{
}
int EXPORT operator <(const StringW & string1, const StringW &string2)
{
	bool s1null = isNullStringW(string1);
	bool s2null = isNullStringW(string2);
	if (s1null && s2null)
		return 0;
	if (s1null && !s2null)
		return 1;
	if (!s1null && s2null)
		return 0;
	require(StrvalidW(string1) && StrvalidW(string2));
	if (string1.count == 0 && string2.count != 0)
		return 1;
	if (string1.count && string2.count == 0)
		return 0;
	return wcscmp(string1.content, string2.content) < 0;
}


int __declspec(naked) leqStringA(StringA  *string1,const StringA *string2)
{
}

int EXPORT operator <= (const StringA & string1, const StringA &string2)
{
	bool s1null = isNullStringA(string1);
	bool s2null = isNullStringA(string2);
	if (s1null && s2null)
		return 1;
	if (s1null && !s2null)
		return 1;
	if (!s1null && s2null)
		return 0;
	require(StrvalidA(string1) && StrvalidA(string2));
	if (string1.count == 0 && string2.count != 0)
		return 1;
	if (string1.count && string2.count == 0)
		return 0;
	int len = string1.count < string2.count ? string1.count : string2.count;
	return memcmp(string1.content, string2.content,len) <= 0;
}

int __declspec(naked) leqStringW(StringW  *string1,const StringW *string2)
{
}

int EXPORT operator <= (const StringW & string1, const StringW &string2)
{
	if (isNullStringW(string1) || isNullStringW(string2))
		return 0;
	require(StrvalidW(string1) && StrvalidW(string2));
	return Strcmp(string1, string2) <= 0;
}


int __declspec(naked) grString(String  *string1,const String *string2)
{
}
int EXPORT operator > (const StringA &string1, const StringA &string2)
{
	if (&string2 == &invalid_stringA || &string1 == &invalid_stringA)
		return 0;
	require(StrvalidA(string1) && StrvalidA(string2));
	return Strcmp(string1, string2) > 0;
}

int EXPORT operator > (const StringW &string1, const StringW &string2)
{
	if (&string2 == &invalid_stringW || &string1 == &invalid_stringW)
		return 0;
	require(StrvalidW(string1) && StrvalidW(string2));
	return Strcmp(string1, string2) > 0;
}

int __declspec(naked) geqString(const String  *string1,const String *string2)
{
}
int EXPORT operator >= (const StringA & string1, const StringA &string2)
{
	if (&string2 == &invalid_stringA || &string1 == &invalid_stringA)
		return 0;
	require(StrvalidA(string1) && StrvalidA(string2));
	return Strcmp(string1, string2) >= 0;
}
int EXPORT operator >= (const StringW & string1, const StringW &string2)
{
	if (&string2 == &invalid_stringW || &string1 == &invalid_stringW)
		return 0;

	require(StrvalidW(string1) && StrvalidW(string2));
	return Strcmp(string1, string2) >= 0;
}

int EXPORT __declspec(naked) notStringp(Stringp s)
{
}
int EXPORT operator !(StringpA s)
{
	if (s.count == 0 && s.content == NULL && s.parent == NULL)
		return 1;
	return 0;
}
int EXPORT operator !(StringpW s)
{
	if (&s == &invalid_stringpW)
		return 1;
	return 0;
}

int EXPORT operator !=(StringpA s,void *pv)
{
	if (pv != NULL)
		return 1;
	if (s.count == 0)
		return 0;
	return 1;
}

int EXPORT __declspec(naked) get_element_string (const String & string, size_t index)
{
}

int EXPORT operator[](const StringA& string, size_t index)
{
	require(StrvalidA(string) && (index <= string.count));
	return string.content[index];
}

int EXPORT operator[](const StringpA& string, size_t index)
{
	require(StrvalidA(*string.parent) && (index <= string.count));
	return string.content[index];
}
int EXPORT operator[](const StringpW& string, size_t index)
{
	require(StrvalidW(*string.parent) && (index <= string.count));
	return string.content[index];
}

int EXPORT operator[](const StringW& string, size_t index)
{
	require(StrvalidW(string) && (index <= string.count));
	return string.content[index];
}

/*------------------------------------------------------------------------
 Procedure:     set_element_string
 Purpose:       Sets the element of "string" at "index" to "new_val".
                This is a more usual way of setting an Element of
                string to a new value "new_val".
                see too item
                see operator[]()
                see operator[]=()
 Input:         string This is the object we are working on.
                index change element at this position.
                new_val set string[index] to this value.
 Output:        returns the old val from position string[index]
 Errors:
------------------------------------------------------------------------*/

/** Do the same as set_item. Just with a more C-ish array notationso
set_item(string, index, val) == string[index] = val
@param string This is a Structure which will get handeled like
an 'normal' array.
@param index change element at this position.
@param new_val The new value of string[index]
@returns The former element at @a index position.

*/
int __declspec(naked) set_element_stringA(StringA &string, size_t index, int new_val)
{
}
int EXPORT operator[]=(StringA & string, size_t index, int new_val)
{
	require(index < string.count);

	string.content[index] = new_val;
	return new_val;
}


int EXPORT operator[]=(StringpA & string, size_t index, int new_val)
{
	require(StrvalidpA(string) && index < string.count);

	string.content[index] = new_val;
	return new_val;
}

int __declspec(naked) set_element_stringW(StringW &string, size_t index, int new_val)
{
}
int EXPORT operator[]=(StringW & string, size_t index, int new_val)
{
	require(StrvalidW(string) && index < string.count);
	string.content[index] = new_val;
	return new_val;
}

int EXPORT operator[]=(StringpW & string, size_t index, int new_val)
{
	require(StrvalidpW(string) && index < string.count);
	string.content[index] = new_val;
	return new_val; // dummy
}

char * operator *(StringA &str)
{
	require(StrvalidA(str) && str.count > 0);
	return &str.content[0];
}

wchar_t * operator *(StringW &str)
{
	require(StrvalidW(str) && str.count > 0);
	return &str.content[0];
}

char * operator *(StringpA &str)
{
	require(StrvalidpA(str) && str.count > 0);
	return &str.content[0];
}

wchar_t * operator *(StringpW &str)
{
	require(StrvalidpW(str) && str.count > 0);
	return &str.content[0];
}


