#include "str.h"
#include "exceptions.h"
#include "str_private.h"

StringA invalid_stringA = {0};
StringW invalid_stringW = {0};
StringpA invalid_stringpA = {0};
StringpW invalid_stringpW = {0};

/* check on validity */
/** Check if a String is valid.
@param string this string will get checked.
@returns if a string is valid or not a string is valid
if it's either of type SMALL or WIDE and can contain elements.
*/
// inline
int  EXPORT StrvalidA(const StringA & string)
{
	return string.content != NULL && string.capacity > string.count;
}

int  EXPORT StrvalidW(const StringW & string)
{
	return string.content != NULL && string.capacity > string.count;
}

int EXPORT StrvalidpA(const StringpA & string)
{
	return	string.parent != NULL &&
			string.content >= string.parent->content &&
			string.content < string.parent->content + string.parent->count;
}

int EXPORT StrvalidpW(const StringpW & string)
{
	return	string.parent != NULL &&
			string.content >= string.parent->content &&
			string.content < string.parent->content + string.parent->count;
}

extern  int   ConstraintFailed( const char *fn, const char *reason, void *reserved );
void Strabort(char *func,int code,char *expr)
{
	char *p,tmp[1024];
	if (code == PRECONDITION_VIOLATION)
		p = "Precondition violation";
	else if (code == POSTCONDITION_VIOLATION)
		p = "Postcondition violation";
	else {
		sprintf(tmp,"Exception 0x%x",code);
		p = tmp;
	}
	if (func[0] == '_')
		func++;
	ConstraintFailed(func,p,NULL);
}

/** Calculate a new capacity based on the current capacity and
needed space.
This functions follows the following idea. If one concatenates strings it's often
a cascade of catenations. It would be quite expensive to resize an Array for each
catenation. Therfor we try to allocate space in chunks. A solution which
has turned out to work nicely in SmallEiffel is therefor used here. Each time
enlargement takes place we do allocate around the double space than before.
@param capacity Current capacity
@param requested_size Requested capacity
@return Either the old capacity if sufficient, the requested size
if some constraints hold or the doubled capacity
*/
size_t calculate_new_capacity (size_t capacity, size_t requested_size)
{
	if (capacity == 0)
		return requested_size;
	if (capacity / 2  > requested_size){
		return requested_size;
	}
	if (capacity >= requested_size){
		return capacity;
	}
	if (capacity * 2 <= requested_size){
		return(requested_size);
	}
	else {
		return(capacity * 2);
	}
}

void * allocate_proper_space(size_t initial_size, unsigned int elementsize)
{
	void *result;

	result = MALLOC(initial_size*elementsize);
	if (result == NULL)
		Strabort("allocate",POSTCONDITION_VIOLATION,NULL);
	return result;
}

