#include "str.h"
#include "exceptions.h"
#include "str_private.h"

/** @a Strtok does work roughly simular to strtok. But with following
enhancements: The matched token is a new allocate @a String and neither
@a string nor @a delimiter are changed. The position of the match,
can be used as starting point for the next call to @a Strtok.
@param string to be tokenized
@param delimiter Elements which do delimit the tokens.
@param *pos in/out, start position in string if a match can be found
incremented to the next Element not beeing part of the delimiter.
@returns a new allocated Substring, of an invalid String if
at the end of @a string.
@pre Strvalid(string) && Strvalid(delimiter))
@pre *pos >= 0 && *pos <= Strlen
@exceptions
GEF_EXCEPTION_PRECONDITION_VIOLATION if one of the conditions
are not meat.
*/
/** Examples see @example examples/strtok_examples.c
*/
StringpA EXPORT overloaded Strtok(StringA *string, const StringA & delimiter)
{
	static StringA lastString;
	void *pcontent;
	StringpA result;

	if (string == NULL) {
		string = &lastString;
		pcontent = NULL;
	}
	else {
		lastString = *string;
		pcontent = lastString.content;
	}

	require(StrvalidA(delimiter));

	pcontent = strtok(pcontent,delimiter.content);
	if (pcontent) {
		result.content = pcontent;
		result.parent = &lastString;
		result.count = strlen(pcontent);
		return result;
	}
	return invalid_stringpA;
}

StringpW EXPORT overloaded Strtok(StringW *string, const StringW & delimiter)
{
	static StringW lastString;
	wchar_t *wdelim;
	void *pcontent;
	StringpW result;

	if (string == NULL) {
		string = &lastString;
		pcontent = NULL;
	}
	else {
		lastString = *string;
		pcontent = lastString.content;
	}

	require(StrvalidW(delimiter));

	pcontent = wcstok(pcontent, delimiter.content);
	if (pcontent) {
		result.count = wcslen(pcontent);
		result.content = pcontent;
		result.parent = &lastString;
		return result;
	}
	return invalid_stringpW;
}


StringpA EXPORT overloaded Strtok(StringA *string, const char *delimiter)
{
	StringA d;

	d.count = strlen(delimiter);
	d.capacity = d.count+1;
	d.content = (void *)delimiter;
	return Strtok(string,d);
}

StringpW EXPORT overloaded Strtok(StringW *string, const wchar_t *delimiter)
{
	StringW d;

	d.count = wcslen(delimiter);
	d.capacity = d.count+1;
	d.content = (void *)delimiter;
	return Strtok(string,d);
}

